#!/bin/sh
#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: pilmos_install.sh 229186 2012-06-08 21:52:21Z pbaksai $"
#
# who       when        what
# --------  ----------  --------------------------------------------------------
# pbaksai   2012/06/08  CCB-00246: Relaxed gcc compiler check.
# pbaksai   2012/05/16  CCB-00246: Created.
#
#*******************************************************************************
#   NAME
#       pilmos_install.sh - Package installation script for pilmosLINUXRH9
#
#   SYNOPSIS
#       pilmos_install.sh [[-c|--compile] [-d|--debug] [-h|--help] [-u|--uninstall] --] [path]
#
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#       0: All OK
#       1: Cancelled due to insufficient conditions
#       2: Aborted due to errors
#
#   CAUTIONS
#       Uses GNU getopt which is incompatible with Mac OsX
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#-------------------------------------------------------------------------------
#
# Standard Initialization BEGIN
#
# Operating System flavors
#
if [ "`uname`" = "Linux" ]; then 
    enable -n echo
    bash=/bin/bash
    xterm=/usr/bin/xterm
    remote=/usr/bin/rsh
    ping=/bin/ping
    echo_option=-n
    count_option=-c
    count=1000
    rt_ini=""
elif [ "`uname`" = "HP-UX" ]; then 
    bash=/usr/bin/bash
    xterm=/usr/bin/X11/xterm
    remote=/usr/bin/remsh
    ping=/etc/ping
    echo_option=-n
    count_option=-n
    count=0
    rt_ini=""
else 
    bash=bash
    remote=remsh
    xterm=xterm
    ping=ping
    echo_option=""
    count_option=-n
    count=0
    rt_ini=""
fi

#
# Name, date and time
#
progname=$0
datestamp=`date +'%Y-%m-%d'`
timestamp=`date +'%H:%M:%S'`

#
# Environment variables
#
user=${USER}
ws=${HOST}
debug=${SHELL_DEBUG}

#
# Constants
#
if [ "${SHELL_DEBUG}" = ""   -o \
     "${SHELL_DEBUG}" = "no" -o \
     "${SHELL_DEBUG}" = "NO" -o \
     "${SHELL_DEBUG}" = "No" ]; then
    debug=0
else
    debug=1
fi

# Standard Initialization END

# Local Initialization BEGIN
UNPACK_DIR=`pwd`
PACKAGE_NAME=pilmosLINUXRH9
# Local Initialization END

# Standard Library BEGIN
HelpAd() {
    echo " "
    echo " To see the Help Screen use "
    echo " $0 -h"
    echo " "
}

Help() {
    echo " "
    echo " Usage: "
    echo " $0 [[-c|--compile] [-d|--debug] [-h|--help] [-u|--uninstall] --] [path]"
    echo "   where: "
    echo "     -c "
    echo "    --compile: recompiles PILMOSt executable"
    echo "     -d "
    echo "    --debug: enables debugging mode"
    echo "     -h "
    echo "    --help: displays this Help Screen"
    echo "     -u "
    echo "    --uninstall: removes the PILMOS package"
    echo "    path: path to the installation directory"
    echo " "
}

CleanUp() {
    echo "===> Cleaning up..."
    echo "     Done cleaning. <==="
    echo " "
}

Debug() {
    if [ $dflag -eq 1 ]; then
        msg=`echo $1 | awk -F '#' '{printf("     |##> DEBUG:   %-51s <##|\n",$1)}'`
        echo "$msg" 1>&2
    fi
}

Error() {
    msg=`echo $1 | awk -F '#' '{printf("     |*** ERROR:   %-51s ***|\n",$1)}'`
    echo "$msg" 1>&2
}

Explain() {
    msg=`echo $1 | awk -F '#' '{printf("     |***          %-51s ***|\n",$1)}'`
    echo "$msg" 1>&2
}

Frame() {
    frm="+--------------------------------------------------------------------+"
    msg=`echo $frm | awk -F '#' '{printf("     %-70s\n",$1)}'`
    echo "$msg" 1>&2
}

Banner() {
    msg=`echo $1 | awk -F '#' '{printf("     |*** %-60s ***|\n",$1)}'`
    echo "$msg" 1>&2
}

Warning() {
    msg=`echo $1 | awk -F '#' '{printf("     |*** WARNING: %-51s ***|\n",$1)}'`
    echo "$msg" 1>&2
}

Exit() {
    CleanUp
    case $1 in
        0)
            Frame
            Banner "END    $progname"
            Frame
            ;;
        1)
            Frame
            Banner "CANCEL $progname"
            Frame
            ;;
        2)
            Frame
            Banner "ABORT  $progname"
            Frame
            ;;
    esac
    exit $1
}
# Standard Library END

# Local Library BEGIN
SafeLink() {
    source=$1
    dest=$2
    Debug "Source     : >$source<"
    Debug "Destination: >$dest<"
    if [ "$source" = "" -o "$dest" = "" ]; then
        Warning "could not link, source and/or destination are/is blank"
        Explain "Source     : >$source<"
        Explain "Destination: >$dest<"
        return 1
    fi
    # Remove existing destination link to package item
    # so that it does not get backed up again.
    if [ -h $dest ]; then
        if [ "`ls -laAB $dest 2>&- | grep $PACKAGE_NAME`" != "" ]; then
            rm -f $dest 2>&-
        fi
    fi
    # Link making a backup of destination if exists
    Debug "ln --suffix=.pilmos -b -f -s $source $dest"
    ln --suffix=.pilmos -b -f -s $source $dest 2>&-
    if [ $? -ne 0 ]; then
        Warning "Error: ln --suffix=.pilmos -b -f -s $source $dest failed"
        Explain "Symbolic link from $source to $dest was not established."
    fi
    # Register in uninstaller macro
    if [ -f $upath ]; then
        # Remove link installed by package
        Debug "echo rm -f $dest 2>&- >> $upath"
        echo "rm -f $dest 2>&-" >> $upath
        # Restore backup made by ln -b above
        Debug "echo mv -f ${dest}.pilmos $dest 2>&- >> $upath"
        echo "mv -f ${dest}.pilmos $dest 2>&-" >> $upath
    fi
    
    return 0
}
# Local Library END

################
# MAIN PROGRAM #
################

# Standard Prologue BEGIN
Frame
Banner "BEGIN  $progname"
Frame

# Check parameters
cflag=0
dflag=0
eflag=0
uflag=0
ipath=""

# Options may be followed by one colon to indicate they have a required argument
# z and zulu are used for testing purposes only.
options=`getopt -q -o cdhuz -a --long compile,debug,help,uninstall,zulu -n $progname -- "$@"`

# Catches invalid options
if [ $? != 0 ]; then
    Warning "unrecognized option(s) ignored"
fi

# Processes valid options
eval set -- "$options"
while [ $# -gt 0 ]; do
    case $1 in
        -c | --compile)
            cflag=1
            ;;
        -d | --debug)
            dflag=1
            Debug "Turned ON."
            ;;
        -h | --help )
            Help
            Exit 0
            ;;
        -u | --uninstall)
            uflag=1
            ;;
        (--)
            shift
            break
            ;;
        (--*)
            # Catches long options that are allowed by getopt but not considered
            # in this case statement.
            Error "invalid long option $1"
            eflag=1
            ;;
        (-?)
            # Catches short options that are allowed by getopt but not considered
            # in this case statement.
            Error "invalid short option $1"
            eflag=1
            ;;
        (*)
            # Catches non-option arguments, to be processed later
            # (for the case when no -- was used to end option's section)
            break
            ;;
    esac
    shift
done

# Processes valid non-option arguments
shift $((OPTIND-1))
case $# in
    (0)
        INSTALL_DIR=$UNPACK_DIR
        ask=1
        ;;
    (*)
        INSTALL_DIR=$1
	GREP=`echo $INSTALL_DIR | grep "^/"`
	if [ $? -ne 0 ]; then
            INSTALL_DIR=${PWD}/${INSTALL_DIR}
        fi
        ask=0
	if [ ! -d $INSTALL_DIR ]; then
            mkdir -p $INSTALL_DIR 2>/dev/null
            if [ $? -ne 0 ]; then
                echo "     Cannot create this directory! Please choose another one...\n"
                INSTALL_DIR=$UNPACK_DIR
                ask=1
            else
                echo "     Created directory $INSTALL_DIR\n"
            fi
	elif [ ! -w $INSTALL_DIR ]; then
	    echo "     You do not appear to have write access to" $INSTALL_DIR "."
	    echo "     Please choose another directory for installation."
            INSTALL_DIR=$UNPACK_DIR
	    ask=1
        fi
        shift
        ;;
esac

# Catches remaining non-option arguments
while [ $# -gt 0 ]; do
    case $1 in
        (*)
            Warning "invalid argument $1 ignored"
            ;;
    esac
    shift
done

if [ $eflag -eq 1 ]; then
    HelpAd
    Exit 1
fi
dflag=$(( dflag || debug ))
# Standard Prologue END

# Main Algorithm BEGIN
# Check package components
if [ ! -f "$progname" ]; then
    echo "     Please run this script from inside the package directory."
    echo "     (cd <some path>/$PACKAGE_NAME/)"
    Exit 1
fi

echo "===> Checking package is present..."
if [ ! -f "${PACKAGE_NAME}.tar.gz" ]; then
    Error   "Package ${PACKAGE_NAME}.tar.gz not found"
    Explain "in $UNPACK_DIR"
    Exit 1
fi
echo "     Done Checking package is present. <==="
echo " "

# Choice of installation directory

if [ $ask -eq 1 ]; then
    echo "===> Choose Installation Directory:"
    echo "     The default is the current directory."
fi

while [ $ask -eq 1 ]; do
    echo "     Please enter the installation directory [" $INSTALL_DIR "] :"
    read answer
    test $answer
    if [ $? -ne 0 ]; then
	ask=0
    else
	absolut=`echo $answer | grep "^/"`
	if [ $? -ne 0 ]; then
	    echo "     Please enter an absolute path (starting with / )."
	elif [ ! -d $answer ]; then
	    echo "     Do you want to create " $answer "? [y]"
	    read makedir
	    test $makedir
	    if [ $? -ne 0 ]; then
		makedir=y
	    fi
	    if [ $makedir = "y" ]; then
		mkdir -p $answer 2>/dev/null
		if [ $? -ne 0 ]; then
		    echo "     Cannot create this directory! Please choose another one...\n"
		else
		    INSTALL_DIR=$answer
		    ask=0
		fi
	    fi
	elif [ ! -w $answer ]; then
	    echo "     You do not appear to have write access to" $answer "."
	    echo "     Please choose another directory for installation."
	else
	    INSTALL_DIR=$answer
	    ask=0
	fi
    fi
done
if [ $ask -eq 1 ]; then
    echo "     Done Choosing Installation Directory. <==="
    echo " "
fi

if [ $uflag -eq 1 ]; then
    echo "===> Un-Installing PILMOS package from $INSTALL_DIR\n"
    if [ -d ${INSTALL_DIR}/${PACKAGE_NAME} ]; then
        cd ${INSTALL_DIR}/${PACKAGE_NAME}
    else
        Error   "package does not exist"
        Explain "${INSTALL_DIR}/${PACKAGE_NAME} not found."
        Explain "First install the package with:"
        Explain "~$> pilmos_install.sh $INSTALL_DIR"
        Exit 1
    fi
    if [ -f ${PACKAGE_NAME}.uninstall ]; then
        if [ $dflag -eq 1 ]; then
            cat ${PACKAGE_NAME}.uninstall
        fi
        # Execute uninstaller macro
        chmod 755 ${PACKAGE_NAME}.uninstall
        ${PACKAGE_NAME}.uninstall
        mv -f ${PACKAGE_NAME}.uninstall ${INSTALL_DIR}/${PACKAGE_NAME}.uninstall.done
        # Remove package directory
        cd ../.
        rm -rf ${INSTALL_DIR}/${PACKAGE_NAME}
        # Remove package compressed files and installer
        # Leave only files distributed to users, if any
        if [ -f ${INSTALL_DIR}/${PACKAGE_NAME}.tar.gz ]; then
            rm -f ${INSTALL_DIR}/${PACKAGE_NAME}.tar.gz 2>&-
            rm -f ${INSTALL_DIR}/README_EXE 2>&-
            rm -f ${INSTALL_DIR}/README_INSTALL 2>&-
            rm -f ${INSTALL_DIR}/pilmos_install.sh 2>&-
        fi
    else
        Error   "package is just unpacked, not installed"
        Explain "${PACKAGE_NAME}.uninstall not found in"
        Explain "${INSTALL_DIR}/${PACKAGE_NAME}"
        Explain "First install the package with:"
        Explain "~$> pilmos_install.sh $INSTALL_DIR"
        Exit 1
    fi
    echo "     Done Un-Installing PILMOS package. <==="
    echo " "
    Exit 0
fi

echo "===> Installing PILMOS package in $INSTALL_DIR\n"

if [ -f "${UNPACK_DIR}/${PACKAGE_NAME}.tar.gz" ]; then
    echo "     ===> Unpacking ${UNPACK_DIR}/${PACKAGE_NAME}.tar.gz to ${INSTALL_DIR}"
    rm -rf ${INSTALL_DIR}/${PACKAGE_NAME} 2>&-
    rm -rf ${INSTALL_DIR}/${PACKAGE_NAME}.uninstall.done 2>&-
    gzip -d -c ${UNPACK_DIR}/${PACKAGE_NAME}.tar.gz | tar xvf - -C $INSTALL_DIR > /dev/null
    echo "          Done Unpacking."
    echo " "
else
    Error   "Package ${PACKAGE_NAME}.tar.gz not found"
    Explain "in $UNPACK_DIR"
    Exit 2
fi
cp -rp ${UNPACK_DIR}/${PACKAGE_NAME}/doc/README_P* ${UNPACK_DIR}/.

# Create uninstaller macro
upath=${INSTALL_DIR}/${PACKAGE_NAME}/${PACKAGE_NAME}.uninstall
echo "# PILMOS Un-Installer created ${datestamp}T${timestamp}" > $upath
echo "cd ${INSTALL_DIR}/${PACKAGE_NAME}" >> $upath

# Compile PILMOS tool
cd ${INSTALL_DIR}/${PACKAGE_NAME}
if [ $cflag -eq 1 ]; then
    echo "     ===> Compiling PILMOS Tool..."
    # Silent compilation option if debuggin is OFF
    co=""
    if [ $dflag -eq 0 ]; then
        # To be defined
        co=""
    fi
    # Check gcc version
    gcc -v 2>&-
    gcc_rev=$?
    if [ $gcc_rev -ne 0 ]; then
        # Error "gcc compiler not found"
        # Exit 2
        Warning "gcc compiler could not be found"
    fi
    cd src
    # 1.- Generate libcfitsio.a
    echo "          ---> Unzipping cfitsio source..."
    gzip -d -c cfitsio.tar.gz | tar xvf - -C . 2>&- > /dev/null
    echo "               Done Unzipping cfitsio source."
    echo " "
    echo "          ---> Compiling cfitsio library..."
    cd PILMOS/cfitsio
    rm -f *.o *.a *.so 2>&-
    ./configure
    # make shared
    make
    # cp libcfitsio.so ../../../lib/. 2>&-
    cp libcfitsio.a ../../../lib/. 2>&-
    rm -f *.o *.a *.so 2>&-
    cd ../../.
    echo "               Done cfitsio library."
    echo " "
    # 2.- Generate libwcs.a
    echo "          ---> Unzipping wcssubs source..."
    gzip -d -c wcssubs.tar.gz | tar xvf - -C . 2>&- > /dev/null
    echo "               Done Unzipping wcssubs source."
    echo " "
    echo "          ---> Compiling wcssubs library..."
    cd PILMOS/wcssubs
    rm -f *.o *.a 2>&-
    make
    cp libwcs.a ../../../lib/. 2>&-
    rm -f *.o *.a 2>&-
    cd ../../.
    echo "               Done wcssubs library."
    echo " "
    # 3.- Generate PILMOSt executable
    echo "          ---> Generating PILMOSt executable..."
    cd pilmost
    rm -f PILMOSt 2>&-
    # gcc -I./../PILMOS/cfitsio -I./../PILMOS/wcssubs -O PILMOSt.c -lm -lnsl ../../lib/libcfitsio.so ../../lib/libwcs.a -o PILMOSt
    gcc -I./../PILMOS/cfitsio -I./../PILMOS/wcssubs -O PILMOSt.c -lm -lnsl ../../lib/libcfitsio.a ../../lib/libwcs.a -o PILMOSt
    cp PILMOSt ../../bin/. 2>&-
    rm -f PILMOSt 2>&-
    echo "               Done PILMOSt executable."
    echo " "
    echo "          Done Compiling PILMOS Tool. <==="
    echo " "
fi

echo "     ===> Installing scripts, libraries and binaries..."
cd ${INSTALL_DIR}/${PACKAGE_NAME}
if [ "$VLTROOT" != "" -a "$INTROOT" != "" ]; then
    # If it was installed in a VLTSW directory structure, replace some libraries
    # and binaries in INTROOT
    vltroot=$VLTROOT
    tcltkroot=$TCLTK_ROOT
    if [ -d $INTROOT/lib ]; then
        libpath=${INTROOT}/lib
        echo "          Linking lib from $libpath..."
        for l in libguidecam.tcl libcfitsio.so libwcs.a; do
            if [ -d $libpath/$l ]; then
                rm -rf $libpath/$l 2>&-
            fi
            SafeLink ${PWD}/lib/$l $libpath/$l
        done
        echo "          Done Linking lib."
        echo " "
    fi
    if [ -d $INTROOT/bin ]; then
        binpath=${INTROOT}/bin
        echo "          Linking bin from $binpath..."
        for b in PILMOSt guidecamLite guidecamMain pilmos.sh; do
            SafeLink ${PWD}/bin/$b $binpath/$b
        done
        echo "          Done Linking bin."
        echo " "
    fi
else
    vltroot=$INSTALL_DIR/${PACKAGE_NAME}
    tcltkroot=$INSTALL_DIR/${PACKAGE_NAME}/tcltk
    # Else, if no VLTSW found, create links to libraries, executables and scripts,
    # Put the links to bin in the first 'bin' found in the PATH
    # for p in `echo $PATH | cut -d':' -f0- | sed 's/:/ /g'`; do
    #     binpath=`echo $p | awk '/\/bin/{printf("%s",$1)}'`
    #     if [ "$binpath" != "" ]; then
    #         if [ -d $binpath ]; then
    #             break
    #         fi
    #     fi
    # done
    if [ "$binpath" == "" ]; then
        mkdir -p ${HOME}/bin
        binpath=${HOME}/bin
    fi
    if [ "$binpath" != "" ]; then
        echo "          Linking bin from $binpath..."
        for b in PILMOSt guidecamLite guidecamMain pilmos.sh seqWish; do
            SafeLink ${INSTALL_DIR}/${PACKAGE_NAME}/bin/$b ${binpath}/$b
        done
        echo "          Done Linking bin."
        echo " "
    else
        Error   "could not define links to executables"
        Explain "Cannot complete install."
        Exit 2
    fi
    # Put the links to lib in the first 'lib' found in the LD_LIBRARY_PATH
    # for p in `echo $LD_LIBRARY_PATH | cut -d':' -f0- | sed 's/:/ /g'`; do
    #     libpath=`echo $p | awk '/\/lib/{printf("%s",$1)}'`
    #     if [ "$libpath" != "" ]; then
    #         if [ -d $libpath ]; then
    #             break
    #         fi
    #     fi
    # done
    if [ "$libpath" == "" ]; then
        mkdir -p ${HOME}/lib
        libpath=${HOME}/lib
    fi
    if [ "$libpath" != "" ]; then
        echo "          Linking lib from $libpath..."
        for l in `ls -1AB ${INSTALL_DIR}/${PACKAGE_NAME}/lib`; do
            SafeLink ${INSTALL_DIR}/${PACKAGE_NAME}/lib/$l ${libpath}/$l
        done
        # Tcl/Tk link to the package tcltk libraries
        for l in `ls -1AB ${INSTALL_DIR}/${PACKAGE_NAME}/tcltk/lib`; do
            SafeLink ${INSTALL_DIR}/${PACKAGE_NAME}/tcltk/lib/$l ${libpath}/$l
        done
        echo "          Done Linking lib."
        echo " "
    else
        Error   "could not define links to libraries"
        Explain "Cannot complete install."
        Exit 2
    fi
fi
# Add extra cleaning of bin and lib links to the uninstaller macro
if [ "$binpath" != "" ]; then
    echo "rm -f \`ls -laAB $binpath 2>&- | grep $PACKAGE_NAME | awk '{printf(\"$binpath/%s \",\$9)}'\`" >> $upath
fi
if [ "$libpath" != "" ]; then
    echo "rm -f \`ls -laAB $libpath 2>&- | grep $PACKAGE_NAME | awk '{printf(\"$libpath/%s \",\$9)}'\`" >> $upath
fi
echo "          Done Installing scripts, libraries and binaries. <==="
echo " "

echo "     ===> Updating launch script..."
# Jejeje, 27 backslashes to replace one / by \\\/ 
# (3 backslashes followed by a slash)
# giving a substitution like /my/full/path/to/pilmos/ for
# \\\/my\\\/full\\\/path\\\/to\\\/pilmos
# which is needed to pass this path string throug sed again without the / being
# mistaken by sed as pattern delimiters.
PILMOS_PATH=`echo ${INSTALL_DIR}/${PACKAGE_NAME} | sed s/[/]/\\\\\\\\\\\\\\\\\\\\\\\\\\\//g`
TCLTK_PATH=`echo ${tcltkroot} | sed s/[/]/\\\\\\\\\\\\\\\\\\\\\\\\\\\//g`
VLT_PATH=`echo ${vltroot} | sed s/[/]/\\\\\\\\\\\\\\\\\\\\\\\\\\\//g`
BIN_PATH=`echo ${binpath} | sed s/[/]/\\\\\\\\\\\\\\\\\\\\\\\\\\\//g`
LIB_PATH=`echo ${libpath} | sed s/[/]/\\\\\\\\\\\\\\\\\\\\\\\\\\\//g`
# Extract binpath if present, then add it again. This way, if it is not present
# then it gets added, else it is not copied. Same for libpath.
CMD="echo $PATH | `echo $BIN_PATH: | awk '{printf(\"sed -e s/%s//g\",$1)}'`"
Debug "CMD BIN = >$CMD<"
SUBPATH=`eval $CMD`
Debug "SUBPATH BIN = >$SUBPATH<"
BINPATH=$binpath:$SUBPATH
Debug "BINPATH = >$BINPATH<"
BIN_PATH=`echo ${BINPATH} | sed s/[/]/\\\\\\\\\\\\\\\\\\\\\\\\\\\//g`
CMD="echo $LD_LIBRARY_PATH | `echo $LIB_PATH: | awk '{printf(\"sed -e s/%s//g\",$1)}'`"
Debug "CMD LIB = >$CMD<"
SUBPATH=`eval $CMD`
Debug "SUBPATH LIB = >$SUBPATH<"
LIBPATH=$libpath:$SUBPATH
Debug "LIBPATH = >$LIBPATH<"
LIB_PATH=`echo ${LIBPATH} | sed s/[/]/\\\\\\\\\\\\\\\\\\\\\\\\\\\//g`
Debug "PILMOS_ROOT = >$PILMOS_PATH<"
Debug "TCLTK_ROOT  = >$TCLTK_PATH<"
Debug "VLTROOT     = >$VLT_PATH<"
Debug "BINPATH     = >$BIN_PATH<"
Debug "LIBPATH     = >$LIB_PATH<"
sed -e s/@@PILMOSBASEPATH@@/$PILMOS_PATH/ \
    -e s/@@TCLTKPATH@@/$TCLTK_PATH/ \
    -e s/@@VLTPATH@@/$VLT_PATH/ \
    -e s/@@BINPATH@@/$BIN_PATH/ \
    -e s/@@LIBPATH@@/$LIB_PATH/ \
    ${INSTALL_DIR}/${PACKAGE_NAME}/src/pilmos.sh.tpl > ${INSTALL_DIR}/${PACKAGE_NAME}/bin/pilmos.sh
chmod 755 ${INSTALL_DIR}/${PACKAGE_NAME}/bin/pilmos.sh
echo "          Done Updating launch script. <==="
echo " "

echo "     ===> Copying configuration files..."
# Create PILMOS user directory
if [ ! -d "${HOME}/.pilmos" ]; then
    mkdir -p ${HOME}/.pilmos
fi
# Backup skycat.cfg
if [ -f "${HOME}/.pilmos/skycat.cfg" ]; then
    mv -f ${HOME}/.pilmos/skycat.cfg ${HOME}/.pilmos/skycat.cfg.$datestamp.$timestamp
fi
# Copy configuration files
cp -p ${INSTALL_DIR}/${PACKAGE_NAME}/config/data_template.fits ${HOME}/.pilmos/.
cp -p ${INSTALL_DIR}/${PACKAGE_NAME}/config/test_catalog.cat ${HOME}/.pilmos/test_cont_cat.dat
cp -p ${INSTALL_DIR}/${PACKAGE_NAME}/config/skycat.cfg ${HOME}/.pilmos/.
cp -p ${INSTALL_DIR}/${PACKAGE_NAME}/bitmaps/skycat-logo.gif ${HOME}/.pilmos/.
echo "rm -f ${HOME}/.pilmos/data_template.fits" >> $upath
echo "rm -f ${HOME}/.pilmos/test_cont_cat.dat" >> $upath
echo "rm -f ${HOME}/.pilmos/skycat.cfg" >> $upath
echo "rm -f ${HOME}/.pilmos/skycat-logo.gif" >> $upath
echo "          Done Copying configuration. <==="
echo " "
echo "     Done Installing PILMOS package. <==="
echo " "

Explain " "
Explain "Execute PILMOS tool with:"
Explain "~$> pilmos.sh "
Explain " "
Explain "Have Fun With It! "
Explain " "
# Main Algorithm END

# Standard Epilogue BEGIN
Exit 0
# Standard Epilogue END

#
#___PILMOS___
