#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: armDisplayDlgLite.tcl 229023 2012-06-03 06:49:07Z pbaksai $"
#
# who       when        what
# --------  ----------  --------------------------------------------------------
# pbaksai   2012/06/02  Added changes made by P. Bristow on 2012/05/14
#                       Improved contributed catalogue widgets layout.
# pbaksai   2012/05/20  Added getContcat method to input contributed catalog
#                       file name interactively.
# pbaksai   2012/05/19  Added curly braces to expressions and ifs
# pbaksai   2012/02/25  Corrected svn_version number extraction
# pbristow  2012/02/01  Field for contributed catalogue, call to PILMOSt
#                       executable, PILMOS parameter name changes, added PPOS
#                       parameter.
# pbaksai   2011/06/18  Added savePosition method for PILMOS
# sskole    31/01/2000  Created
#
#*******************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#-------------------------------------------------------------------------------
#
# guidecam_block begin

itk::usual armDisplayDlgLite {}

class armDisplayDlgLite {
    inherit util::TopLevelWidget

    constructor {args} {}
    destructor {}
    
    public method prepareCatalog {}
    public method starClicked {}
    public method close {} {}
    public method apply {} {}
    public method showCoords {} {}
    public method getContcat {} {}
    public method savePosition {} {}
    public method createWidgets {w} {}
    public method create3Columns {widgetList leftF middleF rightF} {}
    public method create2Columns {widgetlist leftF middleF rightF} {}
    public method getInstance {}
    public method selectionOff {}
    public method selectionOn {}
    public method selectInstrumentMode {instrument mode}

    # updateFields is called by armDisplay to update label values
    public method updateFields {\
	    rotator\
	    adaptor\
	    ttable\
	    starRa\
	    starDec\
	    actTelRa\
	    actTelDec\
	    refTelRa\
	    refTelDec\
	    focusId} {}

    itk_option define -image image Image {}

    # fonts used
    itk_option define -labelfont labelFont LabelFont -Adobe-helvetica-bold-r-normal--12*
    itk_option define -valuefont valueFont ValueFont -Adobe-helvetica-medium-r-normal--12*

    # set the width for displaying labels and values
    itk_option define -labelwidth labelWidth LabelWidth 22
    itk_option define -outputwidth outputWidth OutputWidth 11
    itk_option define -entrywidth entryWidth EntryWidth 12

    private common arm            ""
    private common showArmLast    1 
    private common showDetLast    1
    private common showAdapLast   1
    private common focus          2
    private common simulation     1
    private common imageSvr       "dss@eso"
    private common loadImageLast  1
    private common pickTypeLast   1
    private common instance       ""
    private common instrumentLast "VIMOS"
    private common modeLast       "Vimos"
    private common svn_version    ""
    private variable starRa       ""
    private variable starDec      ""
    private variable probeRa      ""
    private variable probeDec     ""
    private variable contCatFile  ""
    private variable origBindTags ""
    private variable updateRate   1
    private variable vigLim       "NONE"
    private variable detX         0
    private variable detY         0
}

#------------------------------------------------------
# Constructor
#------------------------------------------------------

body armDisplayDlgLite::constructor { args } {
    global gvar ::showArm ::showDet ::showAdap ::loadImage ::env ::pickType 

    # This number goes into PAF file generated for PILMOS. 
    # Version number corresponds to last time *this* source
    # code file was modified, not guidecam module.
    set svn_version [lindex [split [lindex {"@(#) $Id: armDisplayDlgLite.tcl 229023 2012-06-03 06:49:07Z pbaksai $"} 0]] 3]
    eval itk_initialize $args 
    
    wm title $w_ "Guide Probe Settings"
    
    # main controls frame
    pack [set mainf [frame $w_.mainf]] -side top -fill both \
	    -expand 1 -padx 1m -pady 1m -ipadx 1m -ipady 1m
    
    pack [set topf [frame $mainf.topf]] -side top -fill both -expand 1
    
    createWidgets $topf

    if {$arm == ""} {
	set arm [armDisplay $w_.armdisplay -image $itk_option(-image)]
    } 
    set showArm $showArmLast
    set showDet $showDetLast
    set showAdap $showAdapLast   
    set loadImage $loadImageLast
    set pickType $pickTypeLast
    set instance $this
    
    selectInstrumentMode $instrumentLast $modeLast
    # apply
}

body armDisplayDlgLite::destructor {} {

}

body armDisplayDlgLite::selectionOn {} {
    global  objRa objDec

    set canvas [$itk_option(-image) get_canvas]
    set origBindTags [bindtags $canvas]
    set pickTag pick$w_

    bind $pickTag <1> "set objRa \"\" ; set objDec \"\" "
    bind $pickTag <ButtonRelease-1> " [code $this starClicked] "
    bindtags $canvas $pickTag
}

body armDisplayDlgLite::selectionOff {} {

    if {"$origBindTags" != ""} {
	bindtags [$itk_option(-image) get_canvas] $origBindTags
	set origBindTags ""
    }
}

proc armDisplayDlgLite::getInstance {} {return $instance}

body armDisplayDlgLite::starClicked {} {
    global objRa objDec altPosition

    if {"$objRa" != "" && "$objDec" != "" } {
	set msg "Move to the selected guide star?"
	set result [tk_dialog .dialog "Guide Probe" $msg "" 0 Accept Cancel]
	if {$result == 0} {
	    set starRa $objRa
	    set starDec $objDec
	    $itk_component(starRaValue) config -value $starRa
	    $itk_component(starDecValue) config -value $starDec
            $altPosition config -state normal
	    apply
	}
    }
}

body armDisplayDlgLite::apply {} {
    global rotEntry showArm showDet showAdap\
	    refTelRaEntry refTelDecEntry\
	    loadImage pickType altPos pickDlg

    if { $rotEntry != ""} {
	set actualRot [expr {$rotEntry * -1.0}]
    } else {
	set actualRot ""
    }
    
    $arm updateArmData\
	    $instance\
	    $actualRot\
	    ""\
	    ""\
	    $showArm\
	    $showDet\
	    $showAdap\
	    $focus\
	    $simulation\
	    $updateRate\
	    $starRa\
	    $starDec\
	    ""\
	    ""\
	    $refTelRaEntry\
	    $refTelDecEntry\
	    $imageSvr\
	    $loadImage\
	    $vigLim\
	    $altPos\
	    $detX\
	    $detY

    set rotEntry "" 
    set refTelRaEntry ""
    set refTelDecEntry ""

    set showArmLast $showArm
    set showDetLast $showDet
    set showAdapLast $showAdap
    set loadImageLast $loadImage

    if {$pickType == 0} {
	AstroCat::hideCatalogs
	if { [info exists pickDlg] } { 
	    $pickDlg close 
	} else {
	    selectionOff
	}
    } elseif {$pickType == 1} {
	if { [info exists pickDlg] } { 
	    $pickDlg close 
	} else {
	    selectionOff
	}
	after 1000 [code $this prepareCatalog]
	
    } elseif {$pickType == 2} {
	if {![info exists pickDlg]} {
	    AstroCat::hideCatalogs
	    selectionOff
	    set parentWin [winfo parent $w_]
	    set parentObj "::$parentWin"
	    guidecamPickObject $itk_option(-image) $parentWin $parentObj
	    while {1} {
		if { [info exists pickDlg] } {
		    $pickDlg pick_object
		} else {break}
	    }
	}
    }
}

body armDisplayDlgLite::prepareCatalog {} {
    
    set catalog [AstroCat::new_catalog {USNO at ESO} "::$itk_option(-image)" 0]
    update
    selectionOn
    if {[$catalog getListCount] == 0} {
	$catalog search
    }
}

body armDisplayDlgLite::close {} {

    # savePosition
    wm withdraw $w_
}

body armDisplayDlgLite::showCoords {} {
    global probeRaValue probeDecValue ttableValue

    if { $ttableValue == "" } {
	set pposStr ""
    } else {
	if { $ttableValue >= 0 } {
	    set pposStr "PPOS: POS"
	} else {
	    set pposStr "PPOS: NEG"
	}
    }

    set msg "RA: $probeRaValue DEC: $probeDecValue $pposStr"
    set result [tk_dialog .dialog "Guide Star Coords" $msg "" 0 Close]
}

body armDisplayDlgLite::getContcat {} {
    global env contCatValue
    set w [utilNamespaceTail $this]
    set contCatFile [filename_dialog $env(HOME)/.pilmos "*" $w]
    $itk_component(contCatValue) config -value $contCatFile
}

body armDisplayDlgLite::savePosition {} {
    global refTelRaValue refTelDecValue probeRaValue probeDecValue ttableValue rotValue adapValue contCatValue

    if { $contCatValue == "" } {
        set msg "Please supply the path to your contributed catalogue file."
        set result [tk_dialog .dialog "No contributed catalogue filename has been entered" $msg "" 0 CLOSE]
        return
    }
    
    if { $ttableValue == "" } {
	set pposStr ""
	set ppos ""
    } else {
	if { $ttableValue >= 0 } {
	    set pposStr "PPOS: POS"
	    set ppos "POS"
	} else {
	    set pposStr "PPOS: NEG"
	    set ppos "NEG"
	}
    }
    if { [string length $arm] > 0 } {
        set pol1 "SEQ.OPS.PROBEVIG  \"[$arm cget -probeVignet]\"\n"
        set pol2 "SEQ.OPS.DETFOV    \"[$arm cget -detectorFOV]\"\n"
        set pol3 "SEQ.OPS.Q1BOX     \"[$arm cget -chipQ1Box]\"\n"
        set pol4 "SEQ.OPS.Q2BOX     \"[$arm cget -chipQ2Box]\"\n"
        set pol5 "SEQ.OPS.Q3BOX     \"[$arm cget -chipQ3Box]\"\n"
        set pol6 "SEQ.OPS.Q4BOX     \"[$arm cget -chipQ4Box]\"\n"
    } else {
        set pol1 "SEQ.OPS.PROBEVIG  \"\"\n"
        set pol2 "SEQ.OPS.DETFOV    \"\"\n"
        set pol3 "SEQ.OPS.Q1BOX     \"\"\n"
        set pol4 "SEQ.OPS.Q2BOX     \"\"\n"
        set pol5 "SEQ.OPS.Q3BOX     \"\"\n"
        set pol6 "SEQ.OPS.Q4BOX     \"\"\n"
    }
    puts "Probe RA :  $probeRaValue"
    puts "Probe DEC:  $probeDecValue"
    puts "Turn Table: $ttableValue"
    puts "Rotator:    $rotValue"
    puts "Adapter:    $adapValue"
    puts "Contrib. cat. file: $contCatValue"
    puts ""
    puts "Created PAF file 'guidecam.pilmos'"

    # Generate the file for PILMOS here
    append paf "PAF.HDR.START; # Marks start of header\n"
    append paf "PAF.TYPE          \"Instrument Setup\" ; # Type of parameter file\n"
    append paf "PAF.ID            \"VIMOS_PILMOS\" ; # Parameter file ID\n"
    append paf "PAF.NAME          \"guidecam.pilmos\" ; # Parameter file NAME\n"
    append paf "PAF.DESC          \"VIMOS PILMOS Setup File\" ; # Purpose\n"
    append paf "PAF.CRTE.NAME     \"guidecam $svn_version\" ; # SW version date UT\n"
    append paf "PAF.CRTE.DAYTIM   \"[exec date -u +%Y-%m-%dT%H:%M:%S]\" ; # Date and time of creation\n"
    append paf "PAF.LCHG.NAME     \"\" ; # Who did last change\n"
    append paf "PAF.LCHG.DAYTIM   \"\" ; # Date and time of last change\n"
    append paf "PAF.CHCK.NAME     \"\" ; # Appl. checking par. file\n"
    append paf "PAF.CHCK.DAYTIM   \"\" ; # Date and time of lastcheck\n"
    append paf "PAF.CHCK.CHECKSUM \"\" ; # Parameter file checksum\n"
    append paf "PAF.HDR.END; # Marks end of header\n\n"
    append paf "WCS.CRVAL1        $refTelRaValue\n"
    append paf "WCS.CRVAL2        $refTelDecValue\n"
    append paf "ADA.GUID.RA       $probeRaValue\n"
    append paf "ADA.GUID.DEC      $probeDecValue\n"
    append paf "ADA.ABSROT.PPOS   $ppos\n"
    append paf "ADA.POSANG        $rotValue\n"
    append paf "TEL.ADA.POS       $adapValue\n"
    append paf "TEL.TT.POS        $ttableValue\n"
    append paf "PAF.PILMOS.NAME   $contCatValue\n"
    append paf $pol1
    append paf $pol2
    append paf $pol3
    append paf $pol4
    append paf $pol5
    append paf $pol6
    append paf ""
    append paf ""
    set filename "guidecam.pilmos"
    if { [file exists $filename] } {
        set newpath $filename.[clock format [clock seconds] -format %Y%m%dT%T]
        file rename -force $filename $newpath
    }
    set fileId [open guidecam.pilmos w]
    puts $fileId $paf
    ::close $fileId
    exec PILMOSt -a

}

body armDisplayDlgLite::create3Columns {widgetList leftF middleF rightF} {

    foreach widget $widgetList {
	# Create  widgets
	set text [lindex $widget 0]
	set name [lindex $widget 1]

	itk_component add ${name}Label {
	    label $leftF.${name}Label \
		    -text $text\
		    -font $itk_option(-labelfont)
	}
	itk_component add ${name}Entry {
	    entry $middleF.${name}Entry\
		    -textvariable ${name}Entry\
		    -width $itk_option(-entrywidth)\
		    -font $itk_option(-valuefont)
	}

	# bind
	bind $middleF.${name}Entry <KeyPress-Return> [code $this apply]

	itk_component add ${name}Value {
	    LabelValue $rightF.${name}Value \
		    -valuefont $itk_option(-valuefont)\
		    -textvariable ${name}Value\
		    -valuewidth $itk_option(-outputwidth)\
		    -relief groove
	} 
	pack $itk_component(${name}Label) -side top -padx 1.0m -pady 1.2m -anchor w
	pack $itk_component(${name}Entry) -side top -padx 0.5m -pady 0.5m -anchor w
	pack $itk_component(${name}Value) -side top -padx 0.5m -pady 0.5m -anchor w
    }
}

body armDisplayDlgLite::create2Columns {widgetList leftF middleF rightF} {

    foreach widget $widgetList {
	set text [lindex $widget 0]
	set name [lindex $widget 1]
	
	itk_component add ${name}Label {
	    label $leftF.${name}Label\
		    -text $text \
		    -font $itk_option(-labelfont)
	}
	itk_component add ${name}Value {
	    LabelValue $middleF.${name}Value \
		    -valuefont $itk_option(-valuefont)\
		    -valuewidth $itk_option(-outputwidth)\
		    -textvariable ${name}Value\
		    -relief groove
	} 
	itk_component add ${name}Dummy {
	    label $rightF.$name -text "" 
	}
	pack $itk_component(${name}Label) -side top -padx 0.4m -pady 1.2m -anchor w
	pack $itk_component(${name}Value) -side top -pady 0.5m -anchor w
	pack $itk_component(${name}Dummy) -side top -padx 1.0m -pady 1.2m -anchor w
    }
}

body armDisplayDlgLite::createWidgets {w} {
    global ::showArm ::showDet ::showAdap ::loadImage \
	    ::pickType ::altPosition ::altPos ::gvar ::insMode \
	    rotEntry refTelRaEntry refTelDecEntry \
	    rotValue refTelRaValue refTelDecValue
    
    pack [set topFrame [frame $w.topFrame -bd 2]] \
	    -side top -fill both -expand 0

    pack [set lowFrame [frame $w.lowFrame -bd 2]] \
	    -side top -fill both -expand 0
            # Disable alt position until a GS is pickedp

    #===========================================================================
    # Show State frame
    #==========================================================================
    pack [set visibleF [frame $topFrame.visibleF -bd 2 -relief groove]] \
	    -side left -fill both -expand 1 
    pack [label $visibleF.label -text "Show State"] \
	    -side top -padx 1m -pady 2m

    # Create visibility state radio buttons and check box
    pack [checkbutton $visibleF.loadImage -text "Automatic Image Load" -variable loadImage] \
	    -side top -padx 0.5m -pady 0.5m -anchor w
    pack [checkbutton $visibleF.showArm	-text "Show Probe" -variable showArm] \
	    -side top -padx 0.5m -pady 0.5m -anchor w
    pack [checkbutton $visibleF.showDet -text "Show Detector" -variable showDet] \
	    -side top -padx 0.5m -pady 0.5m -anchor w
    pack [checkbutton $visibleF.showAdap -text "Show Fields" -variable showAdap] \
	    -side top -padx 0.5m -pady 0.5m -anchor w  

    #=============================================================================
    # Guide Star frame
    #============================================================================
    pack [set starF [frame $topFrame.starF -bd 2 -relief groove]] \
	   -side left -fill both -expand 1
    pack [label $starF.label -text "Guide Star"] \
	    -side top -padx 1m -pady 2m
    pack [set leftF [frame $starF.leftF -bd 2]] \
	    -side left -fill both -expand 0
    pack [set middleF [frame $starF.middleF -bd 2]] \
	    -side left -fill both -expand 0
    pack [set rightF [frame $starF.rightF -bd 2]] \
	    -side left -fill both -expand 0

    # pack [radiobutton $leftF.off -text "No Selection" -variable pickType -value 0] \
    #     -side top -padx 0.5m -pady 1.0m -anchor w
    pack [radiobutton $leftF.catalog -text "Catalog Selection" -variable pickType -value 1] \
	    -side top -padx 0.5m -pady 1.0m -anchor w
    # pack [radiobutton $leftF.fit -text "Fit Selection" -variable pickType -value 2] \
    #     -side top -padx 0.5m -pady 1.0m -anchor w

    itk_component add starRaValue {
	LabelValue $middleF.starRaValue \
		-text "RA: " \
		-valuefont $itk_option(-valuefont) \
		-valuewidth $itk_option(-outputwidth) \
		-relief groove
    }
    itk_component add starDecValue {
	LabelValue $middleF.starDecValue \
		-text "Dec:" \
		-valuefont $itk_option(-valuefont) \
		-valuewidth $itk_option(-outputwidth) \
		-relief groove
    }
    pack $itk_component(starRaValue) -side top -padx 1m -pady 1.0m -anchor w
    pack $itk_component(starDecValue) -side top -padx 1m -pady 1.0m -anchor w

    # Disable alt position until a GS is picked
    set altPosition $middleF.altPosition
    pack [checkbutton $middleF.altPosition -text "Alternate Position" \
                                           -variable altPos \
                                           -state disabled] \
                                           -side top \
                                           -padx 1m \
                                           -pady 2m \
                                           -anchor w  

    #=============================================================================
    # Instrument Frame
    #============================================================================
    pack [set insF [frame $lowFrame.insF -bd 2 -relief groove]] \
	    -side top -fill both -expand 1
    pack [label $insF.label -text "Instrument"] \
	    -side top -padx 1m -pady 2m
    pack [set topF [frame $insF.topF -bd 2]] \
	    -side top -fill both -expand 0
    pack [set botF [frame $insF.botF -bd 2]] \
	    -side top -fill both -expand 0

    pack [menubutton $topF.selection -text "Instrument Selection" \
                                     -menu $topF.selection.menu \
          ] \
         -side left \
         -padx 3m \
         -pady 1m
    menu $topF.selection.menu
    $topF.selection.menu add cascade -label VIMOS@NaB@UT3 \
                                     -menu $topF.selection.menu.vimos 

    # VIMOS menu
    menu $topF.selection.menu.vimos
    $topF.selection.menu.vimos add radiobutton -label Vimos \
                                               -command [code $this selectInstrumentMode VIMOS Vimos]

    pack [label $topF.insSelectionLabel -text "Currently Selected:" \
	                                -font $itk_option(-labelfont) \
         ] \
	 -side left \
         -padx 3m \
         -pady 1m

    itk_component add insSelectionValue {
        LabelValue $topF.insSelectionValue \
                  -valuefont $itk_option(-valuefont)\
                  -valuewidth $itk_option(-labelwidth)\
                  -relief groove
    }
    pack $itk_component(insSelectionValue) -side left -padx 3m -pady 1m

    #======================================================================
    # Create contributed catalogue fields
    #=========================================================================
    pack [button $botF.contcat -text "Contributed Catalogue" \
	                       -command [code $this getContcat] \
         ] \
         -side left \
         -padx 3m \
         -pady 1m \
         -anchor w
    pack [label $botF.filePath -text "File Path:" \
	                       -font $itk_option(-labelfont) \
         ] \
	 -side left \
         -padx 1m \
         -pady 1m \
         -anchor w
    itk_component add contCatValue {
        LabelValue $botF.contCatValue \
                   -valuefont $itk_option(-valuefont)\
                   -valuewidth 50\
                   -textvariable contCatValue\
                   -relief groove
    } 
    pack $itk_component(contCatValue) -side left \
                                      -pady 1m \
                                      -anchor w
    
    #=============================================================================
    # Telescope Frame
    #============================================================================
    pack [set telF [frame $lowFrame.telF -bd 2 -relief groove]] \
	    -side left -fill both -expand 1
    pack [label $telF.label -text "Telescope"] \
	    -side top -padx 1m -pady 2m
    pack [set leftF [frame $telF.leftF -bd 2]] \
	    -side left -fill both -expand 0
    pack [set middleF [frame $telF.middleF -bd 2]] \
	    -side left -fill both -expand 0
    pack [set rightF [frame $telF.rightF -bd 2]] \
	    -side left -fill both -expand 0
     
    # Create status and entry fields
    set widgetList {{"Preset RA:" refTelRa} {"Preset Dec:" refTelDec} \
	{"Rot on sky (deg):" rot} }
    create3Columns $widgetList $leftF $middleF $rightF
    pack [label $leftF.space -text ""] -side top -padx 1m -pady 2m -anchor w
    pack [label $middleF.space -text "nn:nn:nn.nnn"] -side top -padx 1m -pady 2m -anchor w
    pack [label $rightF.space -text "J2000"] -side top -padx 1m -pady 2m -anchor w
    # set widgetList {{"Offset RA:" actTelRa} {"Offset Dec:" actTelDec}}
    # create3Columns $widgetList $leftF $middleF $rightF

    #=============================================================================
    # Guide Probe Frame
    #============================================================================


    pack [set probeF [frame $lowFrame.probeF -bd 2 -relief groove]] \
	    -side left -fill both -expand 1 -anchor w
    pack [label $probeF.label -text "Guide Probe"] \
	    -side top -padx 1m -pady 2m
    pack [set leftF [frame $probeF.leftF -bd 2]] \
	    -side left -fill both -expand 0
    pack [set middleF [frame $probeF.middleF -bd 2]] \
	    -side left -fill both -expand 0
    pack [set rightF [frame $probeF.rightF -bd 2]] \
	    -side left -fill both -expand 0

    # Create status and entry fields
    set widgetList {{"RA:" probeRa} {"Dec:" probeDec} \
                    {"Adapter (deg):" adap} \
                    {"Turn Table (deg):" ttable} \
                   }
    create2Columns $widgetList $leftF $middleF $rightF
    
    # set widgetList {{"Rot on sky (deg):" rot}}
    # create3Columns $widgetList $leftF $middleF $rightF

    set rotEntry "90.0"  
    set refTelRaEntry "0:00:00.0"
    set refTelDecEntry "0:00:00.0"
    set rotValue "90.0"  
    set refTelRaValue "0:00:00.0"
    set refTelDecValue "0:00:00.0"

    #======================================================================
    # Create buttons
    #=========================================================================
    itk_component add buttons {frame $w_.buttons -borderwidth 2 -relief groove}
    
    pack $itk_component(buttons) -side top -fill x 
    
    itk_component add close {
	button $w_.close -text "Close" -command [code $this close]
    }
    itk_component add savePosition {
	button $w_.save -text "PILMOS" -command [code $this savePosition]
    }
    itk_component add showCoords {
	button $w_.load -text "Get Guide Star Coords" -command [code $this showCoords]
    }
    itk_component add apply {
	button $w_.apply -text "Apply" -command [code $this apply]
    }
    
    pack $itk_component(apply) \
         $itk_component(showCoords) \
         $itk_component(savePosition) \
         $itk_component(close) \
         -side left \
         -expand 1 \
         -padx 2m \
         -pady 2m \
         -ipadx 2m \
         -ipady 1m \
         -in $itk_component(buttons)
}

body armDisplayDlgLite::selectInstrumentMode {instrument mode} {
    global ::gvar

    
    switch $instrument {
	ISAAC {
	    # ISAAC@NaB@UT1
	    set focus 3
	    switch $mode {
		IsaacSw {
		    set detX 150
		    set detY 150
		}
		IsaacLw {
		    set detX 72
		    set detY 72
		}
	    }
	}
	CRIRES {
	    # CRIRES@NaA@UT1
	    set focus 2
	    warning_dialog "\nModes not yet implemented for instrument $instrument!!"
	}
	FLAMES {
	    # FLAMES@NaA@UT2
	    set focus 2
	    warning_dialog "\nModes not yet implemented for instrument $instrument!!"
	}
	FORS2 {
	    # FORS2@Cas@UT2
	    set focus 4
	    set detX 120
	    set detY 408
	}
	UVES {
	    # UVES@NaB@UT2
	    set focus 3
	    switch $mode {
		UVESslit {
		    set detX 60
		    set detY 60
		}
		UVESslicer {
		    set detX 60
		    set detY 60
		}
	    }
	}
	VISITOR {
	    # VISITOR@NaA@UT3
	    set focus 2
	    warning_dialog "\nModes not yet implemented for instrument $instrument!!"
	}
	VISIR {
	    # VISIR@Cas@UT3
	    set focus 4
	    warning_dialog "\nModes not yet implemented for instrument $instrument!!"
	}
	VIMOS {
	    # VIMOS@NaB@UT3
	    set focus 3
	    set detX 960
	    set detY 1080
	}
	NIRMOS {
	    # NIRMOS@NaA@UT4
	    set focus 2
	    warning_dialog "\nModes not yet implemented for instrument $instrument!!"
	}
	SINFONI {
	    # SINFONI@Cas@UT4
	    set focus 4
	    warning_dialog "\nModes not yet implemented for instrument $instrument!!"
	}
	NAOS-CONICA {
	    # NAOS-CONICA@NaB@UT4
	    set focus 2
	    warning_dialog "\nModes not yet implemented for instrument $instrument!!"
	}
    }

    $itk_component(insSelectionValue) config -value "$mode@$instrument"
    set $instrumentLast $instrument
    set $modeLast $mode
}

body armDisplayDlgLite::updateFields {\
	rotator\
	adapter\
	ttable\
	probeRa\
	probeDec\
	actTelRa\
	actTelDec\
	refTelRa\
	refTelDec\
        focusId} {

    # Guide probe
    $itk_component(probeRaValue) config -value $probeRa
    $itk_component(probeDecValue) config -value $probeDec
    $itk_component(rotValue) config -value [format "%.2f" [expr {$rotator * -1.0}] ]
    $itk_component(adapValue) config -value [format "%.2f" [expr {$adapter * -1.0}] ]
    $itk_component(ttableValue) config -value [format "%.2f" [expr {$ttable * -1.0}] ]

    # Telescope
    # $itk_component(actTelRaValue) config -value $actTelRa
    # $itk_component(actTelDecValue) config -value $actTelDec
    $itk_component(refTelRaValue) config -value $refTelRa
    $itk_component(refTelDecValue) config -value $refTelDec

    # PILMOS
    $itk_component(contCatValue) config -value $contCatFile

}

# guidecam_block end
#___oOo___
