#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: guidecamGeom.tcl 228658 2012-05-20 05:33:14Z pbaksai $"
#
# who       when        what
# --------  ----------  --------------------------------------------------------
# pbaksai   2012/05/19  Added curly braces to ifs
# pbaksai   2012/02/25  Replaced 'lset' and 'ne' which are not compatible with
#                       Tcl/Tk 8.3
# pbaksai   2011/06/18  Created for PILMOS
#
#*******************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#-------------------------------------------------------------------------------
#
# Procedures for determining the intersection of two polygons in
# two dimensions.

# Copyright (c) 2002 by Kevin B. Kenny.  All rights reserved.
# Copyright (c) 2011 by Pedro Baksai.  All rights reserved.
# See the file,
# 'http://cvs.sourceforge.net/cgi-bin/viewcvs.cgi/tcllib/tcllib/license.terms'
# for terms and conditions of redistribution

# Kevin B. Kenny.
# ccw --
#
#       Determines the direction of turn when going from a first point
#       through a second to a third.
#
proc ccw { p0 p1 p2 } {

    foreach { x0 y0 } $p0 break
    foreach { x1 y1 } $p1 break
    foreach { x2 y2 } $p2 break

    set dx1 [expr { $x1 - $x0 }]
    set dy1 [expr { $y1 - $y0 }]
    set dx2 [expr { $x2 - $x0 }]
    set dy2 [expr { $y2 - $y0 }]

    set prod1 [expr { $dx1 * $dy2 }]
    set prod2 [expr { $dy1 * $dx2 }]
    if { $prod1 > $prod2 } {
        return 1
    }
    if { $prod1 < $prod2 } {
        return -1
    }
    if { $dx1*$dx2 < 0 || $dy1*$dy2 < 0 } {
        return -1
    }
    if { $dx1*$dx1 + $dy1*$dy1 < $dx2*$dx2 + $dy2*$dy2 } {
        return 1
    }
    return 0
}

# P. Baksai
# seglen --
#       Determines the length of the segment defined by p1 and p2.
#
proc seglen { p1 p2 } {

    foreach { x1 y1 } $p1 break
    foreach { x2 y2 } $p2 break
    set dx [expr {$x2 - $x1}]
    set dy [expr {$y2 - $y1}]
    if { $dx == 0 && $dy == 0 } {
        # p1 and p2 are the same point, hence no segment
        return 0
    } else {
        # return the segment length
        return [expr {sqrt(($dx * $dx) + ($dy * $dy))}]
    }
}
 
# P. Baksai
# distance --
#       Determines the distance from p0 to the segment defined by p1 and p2.
#       The distance is the length of the prependicular projection of p0 on
#       p1-p2 segment.
#
proc distance { p0 p1 p2 } {

    set p {}
    set l [seglen $p1 $p2]
    if { $l == 0 } {
        set d [seglen $p0 $p1]
        set p $p1
        set r 0
    } else {
        foreach { x0 y0 } $p0 break
        foreach { x1 y1 } $p1 break
        foreach { x2 y2 } $p2 break
        set r [expr {(($x0 - $x1)*($x2 - $x1) + ($y0 - $y1)*($y2 - $y1)) /
                     ($l*$l)}]
        if { $r < 0 || $r > 1 } {
            set d -1
        } else {
            set d [expr {abs(($y1 - $y0)*($x2 - $x1) - ($x1 - $x0)*($y2 - $y1)) / $l}]
            set p "[expr {$x1 + $r*($x2 - $x1)}] [expr {$y1 + $r*($y2 - $y1)}]"
        }
    }
    return [list $d $p]
}
 
# Kevin B. Kenny.
# lines'cross --
#
#       Find out whether (and where) two lines cross
#
proc lines'cross { line1 line2 } {
    foreach { xa ya xb yb } $line1 break
    foreach { xc yc xd yd } $line2 break

    set det [expr { ( $xb - $xa ) * ( $yd - $yc )
                   - ( $xd - $xc ) * ( $yb - $ya ) }]
    if { $det == 0 } {                 # line segments are parallel
                                       # or collinear
        return {}
    }
    set nump [expr { ( $xd - $xc ) * ( $yb + $ya )
                     - ( $xb + $xa ) * ( $yd - $yc )
                     + 2. * ( $xc * $yd - $xd * $yc ) }]
    if { abs($nump) > abs($det) } {    # point of intersection is
                                       # outside first line segment
        return
    }
    set numq [expr { ( $xd + $xc ) * ( $yb - $ya )
                    - ( $xb - $xa ) * ( $yd + $yc )
                    + 2. * ( $xb * $ya - $xa * $yb ) }]
    if { abs($numq) > abs($det) } {    # point of intersction is
                                       # outside second line segment
       return
    }
    set p [expr { $nump / $det }]
    set opp [expr { 1. + $p }]
    set omp [expr { 1. - $p }]
    set x [expr { ($xa * $omp + $xb * $opp) / 2. }]
    set y [expr { ($ya * $omp + $yb * $opp) / 2. }]
    return [list $x $y]
}

# Kevin B. Kenny.
# hull2d --
#
#       Computes the convex hull of a set of points in two dimensions.
#
proc hull2d { points } {

    if { [llength $points] < 3 } { return $points }

    set p0 [lindex $points 0]
    foreach { x0 y0 } $p0 break
    set minxpy [expr { $x0 + $y0 }]; set xminxpy $x0; set yminxpy $y0
    set maxxpy [expr { $x0 + $y0 }]; set xmaxxpy $x0; set ymaxxpy $y0
    set minxmy [expr { $x0 - $y0 }]; set xminxmy $x0; set yminxmy $y0
    set maxxmy [expr { $x0 - $y0 }]; set xmaxxmy $x0; set ymaxxmy $y0
    set miny $y0; set iminy 0; set xminy $x0
    set n 0
    foreach p $points {
        foreach { x y } $p break
        if { $y < $miny || ( $y == $miny && $x < $xminy ) } {
            set miny $y; set iminy $n; set xminy $x
        }
        set xpy [expr { $x + $y }]
        if { $xpy < $minxpy } {
            set minxpy $xpy; set xminxpy $x; set yminxpy $y
        }
        if { $xpy > $maxxpy } {
            set maxxpy $xpy; set xmaxxpy $x; set ymaxxpy $y
        }
        set xmy [expr { $x - $y }]
        if { $xmy < $minxmy } {
            set minxmy $xmy; set xminxmy $x; set yminxmy $y
        }
        if { $xmy > $maxxmy } {
            set maxxmy $xmy; set xmaxxmy $x; set ymaxxmy $y
        }
        incr n
    }

    if { $xmaxxpy > $xmaxxmy } {
        set xmaxint $xmaxxmy
    } else {
        set xmaxint $xmaxxpy
    }
    if { $ymaxxpy > $yminxmy } {
        set ymaxint $yminxmy
    } else {
        set ymaxint $ymaxxpy
    }
    if { $xminxpy > $xminxmy } {
        set xminint $xminxpy
    } else {
        set xminint $xminxmy
    }
    if { $yminxpy > $ymaxxmy } {
        set yminint $yminxpy
    } else {
        set yminint $ymaxxmy
    }

    set n 0
    foreach p $points {
        foreach { x y } $p break
        if { $n != $iminy
             && ( $x <= $xminint || $x >= $xmaxint
                  || $y <= $yminint || $y >= $ymaxint ) } {
            lappend pts2 [linsert $p 0 [expr { atan2( $y-$miny, $x-$xminy ) }]]
        }
        incr n
    }
    set pts3 [list [list $xminy $miny]]
    foreach p [lsort -real -index 0 $pts2] {
        foreach { - x y } $p break
        lappend pts3 [list $x $y]
    }
    lappend pts3 [list $xminy $miny]

    set i 0
    set M 2 ; set Mm1 1
    foreach pi $pts3 {
        if { $i < 3 } {
            incr i ; continue
        }
        while { [ccw [lindex $pts3 $M] [lindex $pts3 $Mm1] $pi] >= 0 } {
            incr M -1
            incr Mm1 -1
        }
        incr M
        incr Mm1
        set temp [lindex $pts3 $M]
        # lset pts3 $i $temp
        set pts3 [lreplace $pts3 $i $i $temp]
        # lset pts3 $M $pi
        set pts3 [lreplace $pts3 $M $M $pi]
        set pu $temp
        set partial [lrange $pts3 0 $M]
        lappend partial [list $xminy $miny]
        incr i
    }

    return [lrange $pts3 1 $M]
}


namespace eval polygon {}

# Kevin B. Kenny. & P. Baksai
proc polygon::intersections {p1 p2} {
    set res {}
    set i 0
    set lines2 [sides $p2]
    foreach a [sides $p1] {
        incr i
        set j 0
        foreach b $lines2 {
            incr j
            set point [lines'cross $a $b]
            if { [llength $point] } {
                lappend res $point
                lappend res [list p1${i}xp2${j}]
            }
        }
    }
    return $res
}

# Kevin B. Kenny.
proc polygon::overlap {p1 p2} {
    set res {}
    set p1xp2 [intersections $p1 $p2]
    foreach {point name} $p1xp2 {
        lappend res $point
    }
    #-- collects points of one polygon enclosed in the other
    foreach a [corners $p1] {
        if { [enclosed $p2 $a] } {lappend res $a}
    }
    foreach a [corners $p2] {
        if { [enclosed $p1 $a] } {lappend res $a}
    }
    join [hull2d $res]
}

# Kevin B. Kenny.
proc polygon::center {coords} {
    # returns the x y coordinates of the center
    set xsum 0.0
    set ysum 0.0
    set n    0
    foreach {x y} $coords {
        set xsum [expr {$xsum + $x}]
        set ysum [expr {$ysum + $y}]
        incr n
    }
    list [expr {$xsum/$n}] [expr {$ysum/$n}]
}

# Kevin B. Kenny.
proc polygon::enclosed {coords point} {
    # returns true if point is enclosed by a polygon defined by coords
    set testline [concat $point [center $coords]]
    foreach side [sides $coords] {
        if {[lines'cross $side $testline] != ""} {return 0}
    }
    return 1
}

# P. Baksai.
proc polygon::nearest {coords point {avoid {}}} {
    # returns the nearest side of polygon coords relative to point
    # but excluding zone enclosed by avoid.
    set i 0
    set d 99999999
    set ns {}
    set si 0
    set excl [llength $avoid]
    foreach side [sides $coords] {
        incr i
        foreach {x1 y1 x2 y2} $side break
        foreach {t p} [distance $point "$x1 $y1" "$x2 $y2"] break
        if { $excl && [llength $p] } {
            set zone [enclosed $avoid $p]
        } else {
            set zone 0
        }
        if { ($t >= 0) && ($t <= $d) && !$zone } {
            set d $t
            set ns $side
            set si $i
        }
    }
    # puts "Nearest side is >$si<: >$ns<  at >$d< distance."
    return $si
}

# P. Baksai.
proc polygon::addvertex {coords point {avoid {}}} {
    # returns polygon coords with point added as new vertex to the nearest side
    set poly $coords
    foreach {x y} $point break
    set nsi [nearest $poly $point $avoid]
    # puts "Insert to side >$nsi< ??"
    if { $nsi > 0 } {
        set i [expr {$nsi * 2}]
        set poly [linsert $poly $i $x $y]
        # puts "Inserted >$point< in side $nsi"
    }
    return $poly
}

# Kevin B. Kenny.
proc polygon::sides {coords} {
    foreach {x0 y0} $coords break
    set res {}
    foreach {x y} [lrange $coords 2 end] {
        lappend res [list $x0 $y0 $x $y]
        set x0 $x
        set y0 $y
    }
    lappend res [list $x $y [lindex $coords 0] [lindex $coords 1]]
}

# Kevin B. Kenny.
proc polygon::corners {coords} {
    set res {}
    foreach {x y} $coords {
        lappend res [list $x $y]
    }
    set res
}

#
#___o0o___
