#!/bin/sh
#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: pilmos.sh.tpl 209227 2011-02-01 17:17:46Z pbaksai $"
#
# who       when        what
# --------  ----------  --------------------------------------------------------
# pbaksai   2012/05/16  CCB-00246: Created.
#
#*******************************************************************************
#   NAME
#       pilmos.sh.tpl - Launch script for PILMOS
#
#   SYNOPSIS
#       pilmos.sh [-d|--debug] [-h|--help] [-u|--usage]
#
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#       0: All OK
#       1: Cancelled due to insufficient conditions
#       2: Aborted due to errors
#
#   CAUTIONS
#       Uses GNU getopt which is incompatible with Mac OsX
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#-------------------------------------------------------------------------------
#
# Standard Initialization BEGIN
#
# Operating System flavors
#
if [ "`uname`" = "Linux" ]; then 
    enable -n echo
    bash=/bin/bash
    xterm=/usr/bin/xterm
    remote=/usr/bin/rsh
    ping=/bin/ping
    echo_option=-n
    count_option=-c
    count=1000
    rt_ini=""
elif [ "`uname`" = "HP-UX" ]; then 
    bash=/usr/bin/bash
    xterm=/usr/bin/X11/xterm
    remote=/usr/bin/remsh
    ping=/etc/ping
    echo_option=-n
    count_option=-n
    count=0
    rt_ini=""
else 
    bash=bash
    remote=remsh
    xterm=xterm
    ping=ping
    echo_option=""
    count_option=-n
    count=0
    rt_ini=""
fi

#
# Name, date and time
#
progname=$0
datestamp=`date +'%Y-%m-%d'`
timestamp=`date +'%H:%M:%S'`

#
# Environment variables
#
user=${USER}
ws=${HOST}
debug=${SHELL_DEBUG}

#
# Constants
#
if [ "${SHELL_DEBUG}" = ""   -o \
     "${SHELL_DEBUG}" = "no" -o \
     "${SHELL_DEBUG}" = "NO" -o \
     "${SHELL_DEBUG}" = "No" ]; then
    debug=0
else
    debug=1
fi

# Standard Initialization END

# Local Initialization BEGIN
PACK_DIR=`pwd`
PACKAGE_NAME=pilmosLINUXRH9
# Local Initialization END

# Standard Library BEGIN
HelpAd() {
    echo " "
    echo " To see the Help Screen use "
    echo " $0 -h"
    echo " "
}

Help() {
    echo " "
    echo " Usage: "
    echo " $0 [-d|--debug] [-h|--help] [-u|--usage]"
    echo "   where: "
    echo "     -d "
    echo "    --debug: enables debugging mode"
    echo "     -h "
    echo "    --help "
    echo "     -u "
    echo "    --usage: displays this Help Screen"
    echo " "
}

CleanUp() {
    echo "===> Cleaning up..."
    echo "     Done cleaning. <==="
}

Debug() {
    if [ $dflag -eq 1 ]; then
        msg=`echo $1 | awk -F '#' '{printf("     |##> DEBUG:   %-51s <##|\n",$1)}'`
        echo "$msg" 1>&2
    fi
}

Error() {
    msg=`echo $1 | awk -F '#' '{printf("     |*** ERROR:   %-51s ***|\n",$1)}'`
    echo "$msg" 1>&2
}

Explain() {
    msg=`echo $1 | awk -F '#' '{printf("     |***          %-51s ***|\n",$1)}'`
    echo "$msg" 1>&2
}

Frame() {
    frm="+--------------------------------------------------------------------+"
    msg=`echo $frm | awk -F '#' '{printf("     %-70s\n",$1)}'`
    echo "$msg" 1>&2
}

Banner() {
    msg=`echo $1 | awk -F '#' '{printf("     |*** %-60s ***|\n",$1)}'`
    echo "$msg" 1>&2
}

Warning() {
    msg=`echo $1 | awk -F '#' '{printf("     |*** WARNING: %-51s ***|\n",$1)}'`
    echo "$msg" 1>&2
}

Exit() {
    CleanUp
    case $1 in
        0)
            Frame
            Banner "END    $progname"
            Frame
            ;;
        1)
            Frame
            Banner "CANCEL $progname"
            Frame
            ;;
        2)
            Frame
            Banner "ABORT  $progname"
            Frame
            ;;
    esac
    exit $1
}
# Standard Library END

# Local Library BEGIN
# Local Library END

################
# MAIN PROGRAM #
################

# Standard Prologue BEGIN
Frame
Banner "BEGIN  $progname"
Frame

# Check parameters

dflag=0
eflag=0

# Options may be followed by one colon to indicate they have a required argument
# z and zulu are used for testing purposes only.
options=`getopt -q -o dhuz -a --long debug,help,usage,zulu -n $progname -- "$@"`

# Catches invalid options
if [ $? != 0 ]; then
    Warning "unrecognized option(s) ignored"
fi

# Processes valid options
eval set -- "$options"
while [ $# -gt 0 ]; do
    case $1 in
        -d | --debug)
            dflag=1
            Debug "Turned ON."
            ;;
        -h | --help | -u | --usage)
            Help
            Exit 0
            ;;
        (--)
            shift
            break
            ;;
        (--*)
            # Catches long options that are allowed by getopt but not considered
            # in this case statement.
            Error "invalid long option $1"
            eflag=1
            ;;
        (-?)
            # Catches short options that are allowed by getopt but not considered
            # in this case statement.
            Error "invalid short option $1"
            eflag=1
            ;;
        (*)
            # Catches non-option arguments, to be processed later
            # (for the case when no -- was used to end option's section)
            break
            ;;
    esac
    shift
done

# Processes valid non-option arguments
shift $((OPTIND-1))
case $# in
    (0)
        ;;
    (*)
        ;;
esac

# Catches remaining non-option arguments
while [ $# -gt 0 ]; do
    case $1 in
        (*)
            Warning "invalid argument $1 ignored"
            ;;
    esac
    shift
done

if [ $eflag -eq 1 ]; then
    HelpAd
    Exit 1
fi
dflag=$(( dflag || debug ))
# Standard Prologue END

# Main Algorithm BEGIN

export PATH=@@BINPATH@@
export LD_LIBRARY_PATH=@@LIBPATH@@
export VLTROOT=@@VLTPATH@@
export TCLTK_ROOT=@@TCLTKPATH@@
PILMOS_BASE_PATH=@@PILMOSBASEPATH@@
export GUIDECAM_PLUGIN=${PILMOS_BASE_PATH}/lib/libfcPlugins.tcl/FindingChart_plugin.tcl
# export SKYCAT_CONFIG=file:${PILMOS_BASE_PATH}/config/skycat.cfg
export SKYCAT_CONFIG=file:${HOME}/.pilmos/skycat.cfg
export SKYCAT_LOGO=${HOME}/.pilmos/skycat-logo.gif
export TCS_ENVNAME=wt0tcs
unset MODROOT

Debug "PILMOS_BASE_PATH = >$PILMOS_BASE_PATH<"
Debug "GUIDECAM_PLUGIN  = >$GUIDECAM_PLUGIN<"
Debug "SKYCAT_CONFIG    = >$SKYCAT_CONFIG<"
Debug "SKYCAT_LOGO      = >$SKYCAT_LOGO<"
Debug "TCLTK_ROOT       = >$TCLTK_ROOT<"
Debug "VLTROOT          = >$VLTROOT<"
Debug "PATH             = >$PATH<"
Debug "LD_LIBRARY_PATH  = >$LD_LIBRARY_PATH<"

guidecamMain -agProcName agwsControl -tcsEnvName wt0tcs -tcsProc tifCA -cat 1 &

# Main Algorithm END

# Standard Epilgue BEGIN
Exit 0
# Standard Epilgue END

#
#___PILMOS___
