/*******************************************************************************
# E.S.O. - VLT Project
#
# "@(#) $Id: PILMOSt.c 229186 2012-06-08 21:52:21Z pbaksai $"
#
# who       when        what
# --------  ----------  --------------------------------------------------------
# pbristow  2012/06/08  Corrected cont. cat. end of file detection
# pbristow  2012/05/31  Increased file path length to 256
# pbaksai   2012/05/20  Added this header
# pbristow  2012/02/28  Created
#
#*******************************************************************************
#  NAME
#      PILMOSt.c - Creates a fake pre-image for VMMPS
#
#  SYNOPSIS
#      PILMOSt 
#
#  DESCRIPTION
#
#  ARGUMENTS
#
#  PARAMETERS
#
#  INPUTS
#      data_template.fits  : FITS file containing the minimum FITS header
#      guidecam.pilmos.fits: file produced by guidecam Lite with PILMOS support
#                            containing a description of the VIMOS FOV and the 
#                            Guide Probe vignetting in celestial coordinates.
#
#  OUTPUTS
#      pilmos_A.<quad>_vm_<i>.cat: VIMOS contributed catalogues for detector A
#      pilmos_B.<quad>_vm_<i>.cat: VIMOS contributed catalogues for detector B
#      pilmos_A.<quad>_sim_<i>.fits: VIMOS simulated pre-image for detector A
#      pilmos_B.<quad>_sim_<i>.fits: VIMOS simulated pre-image for detector B
#
#  ENVIRONMENT
#
#  RETURN VALUES
#      SUCCESS: if every command succeded
#      FAILURE: if any command failed
#
#  CAUTIONS
#
#  EXAMPLES
#
#  SEE ALSO
#
#
#*******************************************************************************
0        1         2         3         4         5         6         7         8
123456789 123456789 123456789 123456789 123456789 123456789 123456789 1234567890
*/


#include <math.h>
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include "fitsio.h" 
#include "fitsio2.h" 
#include "PILMOSt.h"
#include "time.h"

/*function to parse the guidecam file keywords and match*/
int parse_gcline(char *name, char*namecheck) {
  int ii, match;
  match=0;
  if (strlen(name)>9) {
    match=1;
    ii=0;
    while (ii<11) {
      if (*(name+ii)!=*(namecheck+ii)) {
	match=0;
      }
      ii++;
    }
  }
  if (match==1) {
    return 1;
  } else {
    return 0;
  }
}

double RAdeg2dec(char *RAdeg) {
/* Convert RA from hr:min:sec to decimal deg */
  int ii=0,col1=0,col2=0;
  double RAdec,dechours,decmins,decsecs;
  char *colon,*shortRA,*hash;
  colon=":";
  hash="#";
  RAdec=dechours=decmins=decsecs=0;
  if (*(RAdeg+ii)==*(hash)) {
    ii=1;
  }
  while (ii<strlen(RAdeg)) {
    if (*(RAdeg+ii)==*(colon)) {
      if (col1==0) {
	col1=ii;
      } else {
	col2=ii;
      }
    }
    ii++;
  }
  dechours=15.0*atof(RAdeg);
  shortRA=(RAdeg+col1+1);
  decmins=atof(shortRA)/4.0;
  shortRA=(RAdeg+col2+1);
  decsecs=atof(shortRA)/240.0;
  RAdec=dechours+decmins+decsecs;

  return RAdec;
}

double DECdeg2dec(char *DECdeg) {
/* Convert DEC from deg:min:sec to decimal deg */
  double addorsub;
  int ii=0,col1=0,col2=0;
  double DECdec,decdegs,decmins,decsecs;
  char *colon,*shortDEC,*minus;
  colon=":";
  minus="-";
  DECdec=decdegs=decmins=decsecs=0;
  if (*(DECdeg)==*(minus)) {
    addorsub=-1.0;
  }
  else {
    addorsub=1.0;
  }
  while (ii<strlen(DECdeg)) {
    if (*(DECdeg+ii)==*(colon)) {
      if (col1==0) {
	col1=ii;
      } else {
	col2=ii;
      }
    }
    ii++;
  }
  decdegs=atof(DECdeg);
  shortDEC=(DECdeg+col1+1);
  decmins=atof(shortDEC)/60.0;
  shortDEC=(DECdeg+col2+1);
  decsecs=atof(shortDEC)/3600.0;
  DECdec=decdegs+addorsub*(decmins+decsecs);
  return DECdec;
}

int main(int argc, char* argv[])
{
  int ii,jj,kk,mm,datamin,datamax,col_cnt,mag_col,row_cnt,col_tot,match,quad;
  int offscl,offscl1,offscl2,ID,endflag,nstep;
  char cont_col_names[10][20]={"ID","RA","DEC","","","","","","",""};
  char cont_col_vals[20][20];
  char col[20];
  char name[20], val[20];
  char *quote="\"";
  char *endquote=" \"";
  double rotangle,targra,targdec,guidra,guiddec,keyval;
  double vigra[40], vigdec[40], gpx_pix, gpy_pix, gpx_pix_next, gpy_pix_next;
  double pi=3.1415926535897931;
  double skyto_alpha[POLY_ORD][POLY_ORD], tosky_alpha[POLY_ORD][POLY_ORD], skyto_beta[POLY_ORD][POLY_ORD], tosky_beta[POLY_ORD][POLY_ORD];
  double maskto_alpha[POLY_ORD][POLY_ORD], tomask_alpha[POLY_ORD][POLY_ORD], maskto_beta[POLY_ORD][POLY_ORD], tomask_beta[POLY_ORD][POLY_ORD];
  char *GuideCamFile="guidecam.pilmos";
  char cont_cat_file_name[256];
  char guidppos[4];
  char KW[32];
  long naxes[2]={0};
  fitsfile *fptr,*fptr1;
  long fpixel=1, naxis=2;
  char card[FLEN_CARD];
  int status=0, status1 = 0,  nkeys;  /* MUST initialize status */
  char outstring[400];
  double RAsource, DECsource, x_basic, y_basic, x_dist, y_dist, x_therm, y_therm, x_frac, y_frac;
  double x_basic_vig1, y_basic_vig1, x_dist_vig1, y_dist_vig1;
  double x_basic_vig2, y_basic_vig2, x_dist_vig2, y_dist_vig2;
  double deltax, deltay, gpx_interp, gpy_interp;
  double x_interp, y_interp, x_interp_mirror, corner_1x, corner_1y, corner_2x, corner_2y;
  double mag_in, flux, fluxmin;
  double lst,utc;
  double obs_date_s;
  double exptime=-1.0;
  char sys_date_str[22];
  int Minute,Hour,Day,Month,Year;
  float Second;
  struct tm *Sys_T = NULL;                      
  time_t Tval = 0;     
  char OBJECT[7]="PILMOS"; /*Maybe this should be an input??*/
  float dummyval=0.0;
  double field_radius=0.3;
  double ra_up,ra_down,dec_up,dec_down;
  double ** simple_array_float=NULL;
  int ** sim_array=NULL;
  char head_chars[256];
  char last_head_char;

  struct WorldCoor *vimoswcs;

  char simname[30];
  char vm_cat_file_name[30];
  int filecheck=0;
  int simcount=1;

  FILE *vm_cat_file;
  FILE *cont_cat_file;
  FILE *gcfile= fopen(GuideCamFile, "r");
  if (gcfile==NULL) {
    printf("Error opening guidecam file");
    exit(1);
  }
  else {
    printf("Guidecam File found\n");
    /*Get all necesary parameters that were written to the guidecamfile*/

    /*COULD BE A PROBLEM IF THESE ARE NOT IN THE EXPECTED ORDER...*/
    while(feof(gcfile)== 0){
      fscanf(gcfile,"%s",&name);
      //match=parse_gcline(name,"TEL.TARG.RA");
      match=parse_gcline(name,"WCS.CRVAL1");
      if (match!=0) {
	fscanf(gcfile,"%s",&val);
	targra=RAdeg2dec(val);
      }
      //match=parse_gcline(name,"TEL.TARG.DEC");
      match=parse_gcline(name,"WCS.CRVAL2");
      if (match!=0) {
	fscanf(gcfile,"%s",&val);
	targdec=DECdeg2dec(val);
      }
      //match=parse_gcline(name,"TEL.GS1.RA");
      match=parse_gcline(name,"ADA.GUID.RA");
      if (match!=0) {
	fscanf(gcfile,"%s",&val);
	guidra=RAdeg2dec(val);
      }
      //match=parse_gcline(name,"TEL.GS1.DEC");
      match=parse_gcline(name,"ADA.GUID.DEC");
      if (match!=0) {
	fscanf(gcfile,"%s",&val);
	guiddec=DECdeg2dec(val);
      }
      //match=parse_gcline(name,"TEL.ROT.ANGLE");
      match=parse_gcline(name,"ADA.POSANG");
      if (match!=0) {
	fscanf(gcfile,"%s",&val);
	rotangle=atof(val);
      }
      match=parse_gcline(name,"ADA.ABSROT.PPOS");
      if (match!=0) {
	fscanf(gcfile,"%s",&guidppos);
      }
      match=parse_gcline(name,"PAF.PILMOS.NAME");
      if (match!=0) {
	fscanf(gcfile,"%s",&val);
	strcpy(cont_cat_file_name,val);
      }
      match=parse_gcline(name,"SEQ.OPS.PROBEVIG");
      if (match!=0) {
	for (ii=0;ii<40;ii++) {
	  vigra[ii]=0;
	  vigdec[ii]=0;
	}
	ii=0;
	fscanf(gcfile,"%s",&val);
	if (*(val)==*(quote+0) && *(val+1)==*(quote+0)) {
	  endflag=1;
	} else {
	  endflag=0;
	}
	while (endflag==0) {
	  //	  printf("%s \n",val);
	  if ((float)(ii/2)==(float)(ii)/2.0) {
	    if (*(val)==*(quote+0)) {
	      *(val)=' ';
	    }
	    vigra[ii/2]=RAdeg2dec(val);
	  } else {
	    if (*(val+strlen(val)-1)==*(endquote+0)) {
	      endflag=1;
	    }
	    vigdec[ii/2]=DECdeg2dec(val);
	  }
	  ii++;
	  fscanf(gcfile,"%s",&val);
	}
      }
    }
    fclose(gcfile);
  }
  //printf("Guidecam file processed - rot angle: %g ; target RA: %lf ; target DEC: %lf ; guide star RA %lf ; guide star DEC: %lf ; guide probe orientation: %s ; Contributed catalogue file name: %s \n",rotangle, targra, targdec, guidra, guiddec, guidppos, cont_cat_file_name);

  /* mem allocation for float array*/
  if ((simple_array_float=(double **)calloc(SIZE_B,sizeof(double *))) == NULL)
    {
      printf("ERROR allocating memory for simple_array_float\n");
      return 1;
    }
  for (ii = 0; ii < SIZE_B; ii++)
    {
      if ((simple_array_float[ii]=(double *)calloc((SIZE_A),sizeof(double))) == NULL)
	{
	  printf("ERROR allocating memory for simple_array_float\n");
	  return 1;
	}
    }

  /* mem allocation for int array*/
  if ((sim_array=(int **)calloc(SIZE_B,sizeof(int *))) == NULL)
    {
      printf("ERROR allocating memory for sim_array\n");
      return 1;
    }
  for (ii = 0; ii < SIZE_B; ii++)
    {
      if ((sim_array[ii]=(int *)calloc((SIZE_A),sizeof(int))) == NULL)
	{
	  printf("ERROR allocating memory for sim_array\n");
	  return 1;
	}
    }

  naxes[0]=SIZE_A;
  naxes[1]=SIZE_B;
   /* making cfitsio-friendly arrays */
  for( ii=1; ii<naxes[1]; ii++ )
    simple_array_float[ii] = simple_array_float[ii-1] + naxes[0];
  for( ii=1; ii<naxes[1]; ii++ )
    sim_array[ii] = sim_array[ii-1] + naxes[0];

  /*Loop over the four quadrants*/
  for (quad=1; quad<5; quad++) {
    printf("processing quadrant %d\n", quad); 
    /*reset arrays*/
    for (ii=0; ii<naxes[1]; ii++) {
      for (jj=0; jj<naxes[0]; jj++) {
	simple_array_float[ii][jj]=0.0;
	simple_array_float[ii][jj]=0.0;
	sim_array[ii][jj]=0;
      }
    }
    /*Set filenames according to quadrant, the "!" overwrites any existing file*/
    simcount=1;
    filecheck=0;
    while (filecheck==0) {
      if (quad==1 || quad==4) {
	sprintf(simname,"!%s_B.%d_sim_%d.fits",sim_root_name,quad,simcount);
	sprintf(vm_cat_file_name,"%s_B.%d_vm_%d.cat",sim_root_name,quad,simcount);
      } else {
	sprintf(simname,"!%s_A.%d_sim_%d.fits",sim_root_name,quad,simcount);
	sprintf(vm_cat_file_name,"%s_A.%d_vm_%d.cat",sim_root_name,quad,simcount);
      }
      if (vm_cat_file=fopen(vm_cat_file_name,"r")) {
	fclose(vm_cat_file);
	simcount++;
      }
      else {
	filecheck=1;
      }
    }
    /*Create the fits output file*/
    fits_create_file(&fptr, simname, &status);
    if (status != 0) {
      printf("ERROR: fits_create_file, status=%d\n", status);
      exit(0);
    }
    fits_create_img(fptr, -32, naxis, naxes, &status);
    if (status != 0) {
      printf("ERROR: fits_create_img, status=%d\n", status);
      exit(0);
    }
    /*Open the VIMOS template file*/
    fits_open_file(&fptr1, "~/.pilmos/data_template.fits", READONLY, &status);
    if (status!=0) {
      fprintf(stderr,"Couldn't find the data_template file (data_template.fits, should come with this Guidecam distribution). \n");
      exit(0);
    }

    fits_get_hdrspace(fptr1, &nkeys, NULL, &status);

    /* Read template file and copy its header to the new output file*/
    /*First 9 basic FITS header rows are already present, so skip them*/
    for (ii = 9; ii <= nkeys; ii++)  {
      fits_read_record(fptr1, ii, card, &status1); /* read keyword */
      fits_write_record(fptr, card, &status);
    }
    fits_close_file(fptr1, &status1);

    /*Now fill the header with values from the guidecam file and the sky2ccd and mask2ccd cmfs*/
    fits_update_key(fptr, TDOUBLE, "CRVAL1", &targra, NULL, &status);
    fits_update_key(fptr, TDOUBLE, "CRVAL2", &targdec, NULL, &status);
    fits_update_key(fptr, TDOUBLE, "RA", &targra, NULL, &status);
    fits_update_key(fptr, TDOUBLE, "DEC", &targdec, NULL, &status);
    keyval=SCALE*cos(rotangle*pi/180.0);
    fits_update_key(fptr, TDOUBLE, "CD1_1", &keyval, NULL, &status);
    keyval=-SCALE*sin(rotangle*pi/180.0);
    fits_update_key(fptr, TDOUBLE, "CD2_1", &keyval, NULL, &status);
    keyval=SCALE*sin(rotangle*pi/180.0);
    fits_update_key(fptr, TDOUBLE, "CD1_2", &keyval, NULL, &status);
    keyval=SCALE*cos(rotangle*pi/180.0);
    fits_update_key(fptr, TDOUBLE, "CD2_2", &keyval, NULL, &status);
    fits_update_key(fptr, TDOUBLE, "HIERARCH ESO ADA POSANG", &rotangle, NULL, &status);
    keyval=SCALE*3600.0;
    fits_update_key(fptr, TDOUBLE, "HIERARCH ESO INS PIXSCALE", &keyval, NULL, &status);
    keyval=CRPIX1[quad-1]-50.0;
    fits_update_key(fptr, TDOUBLE, "CRPIX1", &keyval, NULL, &status);
    keyval=CRPIX2[quad-1];
    fits_update_key(fptr, TDOUBLE, "CRPIX2", &keyval, NULL, &status);
    fits_update_key(fptr, TDOUBLE, "HIERARCH ESO ADA GUID RA", &guidra, NULL, &status);
    fits_update_key(fptr, TDOUBLE, "HIERARCH ESO ADA GUID DEC", &guiddec, NULL, &status);
    fits_update_key(fptr, TSTRING, "HIERARCH ESO ADA ABSROT PPOS", &guidppos, NULL, &status);
    keyval=0.0;
    fits_update_key(fptr, TDOUBLE, "HIERARCH ESO ADA ABSROT START", &keyval, NULL, &status);//not used
    fits_update_key(fptr, TDOUBLE, "HIERARCH ESO ADA ABSROT END", &keyval, NULL, &status);//not used
    /*sky2ccd and mask2ccd coeffs are stored in the PILMOSt.h file for all quadrants,
      load the ones for the current quadrant*/
    switch(quad) {
    case 1 : {
      for (ii=0; ii<POLY_ORD; ii++) {
	for (jj=0; jj<POLY_ORD; jj++) {
	  tosky_alpha[ii][jj]=q1tosky_alpha[ii][jj];
	  skyto_alpha[ii][jj]=skytoq1_alpha[ii][jj];
	  tosky_beta[ii][jj]=q1tosky_beta[ii][jj];
	  skyto_beta[ii][jj]=skytoq1_beta[ii][jj];
	  tomask_alpha[ii][jj]=q1tomask_alpha[ii][jj];
	  maskto_alpha[ii][jj]=masktoq1_alpha[ii][jj];
	  tomask_beta[ii][jj]=q1tomask_beta[ii][jj];
	  maskto_beta[ii][jj]=masktoq1_beta[ii][jj];
	}
      }
      break;
    }
    case 2 : {
      for (ii=0; ii<POLY_ORD; ii++) {
	for (jj=0; jj<POLY_ORD; jj++) {
	  tosky_alpha[ii][jj]=q2tosky_alpha[ii][jj];
	  skyto_alpha[ii][jj]=skytoq2_alpha[ii][jj];
	  tosky_beta[ii][jj]=q2tosky_beta[ii][jj];
	  skyto_beta[ii][jj]=skytoq2_beta[ii][jj];
	  tomask_alpha[ii][jj]=q2tomask_alpha[ii][jj];
	  maskto_alpha[ii][jj]=masktoq2_alpha[ii][jj];
	  tomask_beta[ii][jj]=q2tomask_beta[ii][jj];
	  maskto_beta[ii][jj]=masktoq2_beta[ii][jj];
	}
      }
      break;
    }
    case 3 : {
      for (ii=0; ii<POLY_ORD; ii++) {
	for (jj=0; jj<POLY_ORD; jj++) {
	  tosky_alpha[ii][jj]=q3tosky_alpha[ii][jj];
	  skyto_alpha[ii][jj]=skytoq3_alpha[ii][jj];
	  tosky_beta[ii][jj]=q3tosky_beta[ii][jj];
	  skyto_beta[ii][jj]=skytoq3_beta[ii][jj];
	  tomask_alpha[ii][jj]=q3tomask_alpha[ii][jj];
	  maskto_alpha[ii][jj]=masktoq3_alpha[ii][jj];
	  tomask_beta[ii][jj]=q3tomask_beta[ii][jj];
	  maskto_beta[ii][jj]=masktoq3_beta[ii][jj];
	}
      }
      break;
    }
    case 4 : {
      for (ii=0; ii<POLY_ORD; ii++) {
	for (jj=0; jj<POLY_ORD; jj++) {
	  tosky_alpha[ii][jj]=q4tosky_alpha[ii][jj];
	  skyto_alpha[ii][jj]=skytoq4_alpha[ii][jj];
	  tosky_beta[ii][jj]=q4tosky_beta[ii][jj];
	  skyto_beta[ii][jj]=skytoq4_beta[ii][jj];
	  tomask_alpha[ii][jj]=q4tomask_alpha[ii][jj];
	  maskto_alpha[ii][jj]=masktoq4_alpha[ii][jj];
	  tomask_beta[ii][jj]=q4tomask_beta[ii][jj];
	  maskto_beta[ii][jj]=masktoq4_beta[ii][jj];
	}
      }
      break;
    }
    }
    /*Write the coeffs to the fake pre-image header*/
    for (ii=0; ii<POLY_ORD; ii++) {
      for (jj=0; jj<POLY_ORD; jj++) {
	sprintf(KW,"HIERARCH ESO PRO CCD SKY X_%d_%d",ii,jj);
	fits_update_key(fptr, TDOUBLE, KW, &tosky_alpha[ii][jj], NULL, &status);
	sprintf(KW,"HIERARCH ESO PRO CCD SKY Y_%d_%d",ii,jj);
	fits_update_key(fptr, TDOUBLE, KW, &tosky_beta[ii][jj], NULL, &status);
	sprintf(KW,"HIERARCH ESO PRO SKY CCD X_%d_%d",ii,jj);
	fits_update_key(fptr, TDOUBLE, KW, &skyto_alpha[ii][jj], NULL, &status);
	sprintf(KW,"HIERARCH ESO PRO SKY CCD Y_%d_%d",ii,jj);
	fits_update_key(fptr, TDOUBLE, KW, &skyto_beta[ii][jj], NULL, &status);

	sprintf(KW,"HIERARCH ESO PRO CCD MASK X_%d_%d",ii,jj);
	fits_update_key(fptr, TDOUBLE, KW, &tomask_alpha[ii][jj], NULL, &status);
	sprintf(KW,"HIERARCH ESO PRO CCD MASK Y_%d_%d",ii,jj);
	fits_update_key(fptr, TDOUBLE, KW, &tomask_beta[ii][jj], NULL, &status);
	sprintf(KW,"HIERARCH ESO PRO MASK CCD X_%d_%d",ii,jj);
	fits_update_key(fptr, TDOUBLE, KW, &maskto_alpha[ii][jj], NULL, &status);
	sprintf(KW,"HIERARCH ESO PRO MASK CCD Y_%d_%d",ii,jj);
	fits_update_key(fptr, TDOUBLE, KW, &maskto_beta[ii][jj], NULL, &status);
      }
    }

    /*write the sky2ccd diagnostics to the fake pre-image header*/
    fits_update_key(fptr, TDOUBLE, "HIERARCH ESO PRO CCD SKY XRMS",&tosky_xrms[quad-1], NULL, &status);
    fits_update_key(fptr, TDOUBLE, "HIERARCH ESO PRO CCD SKY YRMS",&tosky_yrms[quad-1], NULL, &status);
    fits_update_key(fptr, TDOUBLE, "HIERARCH ESO PRO SKY CCD XRMS",&skyto_xrms[quad-1], NULL, &status);
    fits_update_key(fptr, TDOUBLE, "HIERARCH ESO PRO SKY CCD YRMS",&skyto_yrms[quad-1], NULL, &status);
    fits_update_key(fptr, TDOUBLE, "HIERARCH ESO PRO CCD SKY TEMP",&sky_temp[quad-1], NULL, &status);
    fits_update_key(fptr, TSTRING, "HIERARCH ESO PRO CCD SKY DAYTIM",&sky_daytim[quad-1], NULL, &status);
    fits_update_key(fptr, TSTRING, "HIERARCH ESO OCS CMF FILE1 NAME",&skyccd_file[quad-1], NULL, &status);
    fits_update_key(fptr, TSTRING,"HIERARCH ESO OCS CMF FILE1 CRTEDATE",&sky_crte_daytim[quad-1], NULL, &status);

    /*write the mask linear terms to the fake pre-image header*/
    fits_update_key(fptr, TDOUBLE,"HIERARCH ESO PRO MASK CCD X0",&maskto_x0[quad-1], NULL, &status);
    fits_update_key(fptr, TDOUBLE,"HIERARCH ESO PRO MASK CCD XX",&maskto_xx[quad-1], NULL, &status);
    fits_update_key(fptr, TDOUBLE,"HIERARCH ESO PRO MASK CCD XY",&maskto_xy[quad-1], NULL, &status);
    fits_update_key(fptr, TDOUBLE,"HIERARCH ESO PRO MASK CCD Y0",&maskto_y0[quad-1], NULL, &status);
    fits_update_key(fptr, TDOUBLE,"HIERARCH ESO PRO MASK CCD YX",&maskto_yx[quad-1], NULL, &status);
    fits_update_key(fptr, TDOUBLE,"HIERARCH ESO PRO MASK CCD YY",&maskto_yy[quad-1], NULL, &status);
    fits_update_key(fptr, TDOUBLE,"HIERARCH ESO PRO CCD MASK X0",&tomask_x0[quad-1], NULL, &status);
    fits_update_key(fptr, TDOUBLE,"HIERARCH ESO PRO CCD MASK XX",&tomask_xx[quad-1], NULL, &status);
    fits_update_key(fptr, TDOUBLE,"HIERARCH ESO PRO CCD MASK XY",&tomask_xy[quad-1], NULL, &status);
    fits_update_key(fptr, TDOUBLE,"HIERARCH ESO PRO CCD MASK Y0",&tomask_y0[quad-1], NULL, &status);
    fits_update_key(fptr, TDOUBLE,"HIERARCH ESO PRO CCD MASK YX",&tomask_yx[quad-1], NULL, &status);
    fits_update_key(fptr, TDOUBLE,"HIERARCH ESO PRO CCD MASK YY",&tomask_yy[quad-1], NULL, &status);
    /*write the mask2ccd diagnostics to the fake pre-image header*/
    fits_update_key(fptr, TDOUBLE, "HIERARCH ESO PRO CCD MASK XRMS",&tomask_xrms[quad-1], NULL, &status);
    fits_update_key(fptr, TDOUBLE, "HIERARCH ESO PRO CCD MASK YRMS",&tomask_yrms[quad-1], NULL, &status);
    fits_update_key(fptr, TDOUBLE, "HIERARCH ESO PRO MASK CCD XRMS",&maskto_xrms[quad-1], NULL, &status);
    fits_update_key(fptr, TDOUBLE, "HIERARCH ESO PRO MASK CCD YRMS",&maskto_yrms[quad-1], NULL, &status);
    fits_update_key(fptr, TDOUBLE, "HIERARCH ESO PRO CCD MASK TEMP",&mask_temp[quad-1], NULL, &status);
    fits_update_key(fptr, TSTRING, "HIERARCH ESO PRO CCD MASK DAYTIM",&mask_daytim[quad-1], NULL, &status);
    fits_update_key(fptr, TSTRING, "HIERARCH ESO OCS CMF FILE2 NAME",&maskccd_file[quad-1], NULL, &status);
    fits_update_key(fptr, TSTRING, "HIERARCH ESO OCS CMF FILE2 CRTEDATE",&mask_crte_daytim[quad-1], NULL, &status);

    fits_update_key(fptr, TINT, "HIERARCH ESO OCS CON QUAD",&quad, NULL, &status);
    /*Set the fake pre-image channel temperature to be the same as that of the
      mask2ccd calibration exposure quadrant temperature so that no further
      temperature scaling is performed later by vmmcsPix2mm*/
    sprintf(KW,"HIERARCH ESO INS FOCU%d TEMP",quad);
    fits_update_key(fptr, TDOUBLE, KW,&mask_temp[quad-1], NULL, &status);
    fits_update_key(fptr, TDOUBLE, "HIERARCH ESO TEL AMBI TEMP",&mask_temp[quad-1], NULL, &status);
    fits_update_key(fptr, TDOUBLE, "HIERARCH ESO TEL AMBI TEMP",&mask_temp[quad-1], NULL, &status);
    fits_update_key(fptr, TSTRING, "OBJECT",&OBJECT, NULL, &status);
    fits_update_key(fptr, TSTRING, "HIERARCH ESO OBS TARG NAME",&OBJECT, NULL, &status);
    fits_update_key(fptr, TDOUBLE, "EXPTIME",&exptime, NULL, &status);

    /*get the system time and write it to the file date keyword*/
    Tval = time(NULL);
    Sys_T = localtime(&Tval);
    Second=Sys_T->tm_sec;
    Minute=Sys_T->tm_min;
    Hour=Sys_T->tm_hour;
    Day=Sys_T->tm_mday;
    Month=Sys_T->tm_mon+1;
    Year=1900 + Sys_T->tm_year;

    sprintf(sys_date_str,"%d-",Year);
    if (Month<10) {
      sprintf(sys_date_str,"%s0%d-",sys_date_str,Month);
    } else {
      sprintf(sys_date_str,"%s%d-",sys_date_str,Month);
    }
    if (Day<10) {
      sprintf(sys_date_str,"%s0%dT",sys_date_str,Day);
    } else {
      sprintf(sys_date_str,"%s%dT",sys_date_str,Day);
    }
    if (Hour<10) {
      sprintf(sys_date_str,"%s0%d:",sys_date_str,Hour);
    } else {
      sprintf(sys_date_str,"%s%d:",sys_date_str,Hour);
    }
    if (Minute<10) {
      sprintf(sys_date_str,"%s0%d:",sys_date_str,Minute);
    } else {
      sprintf(sys_date_str,"%s%d:",sys_date_str,Minute);
    }
    if (Second<10.0) {
      sprintf(sys_date_str,"%s0%2.1f",sys_date_str,Second);
    } else {
      sprintf(sys_date_str,"%s%3.1f",sys_date_str,Second);
    }
    fits_update_key(fptr, TSTRING, "DATE", &sys_date_str, NULL, &status);

    /*Write an obviously bogus observation date*/
    obs_date_s=41084.291666667;
    fits_update_key(fptr, TDOUBLE, "MJD-OBS", &obs_date_s, NULL, &status);
    fits_modify_comment(fptr, "MJD-OBS", "Obs start 1971-05-13T07:00:00",&status);
    fits_update_key(fptr, TSTRING, "DATE-OBS", "1971-05-13T07:00:00", NULL, &status);
    fits_update_key(fptr, TSTRING, "HIERARCH ESO OBS START", "1971-05-13T07:00:00", NULL, &status);
    fits_update_key(fptr, TSTRING, "HIERARCH ESO TPL START", "1971-05-13T07:00:00", NULL, &status);
    lst=25200.0;
    fits_update_key(fptr, TDOUBLE, "LST", &lst, NULL, &status);
    utc=3600.0;
    fits_update_key(fptr, TDOUBLE, "UTC", &utc, NULL, &status);

    fits_update_key(fptr, TSTRING, "OBSERVER","PILMOS", NULL, &status);

    /*fill some arbitrary values*/
    dummyval=0.8;
    fits_update_key(fptr, TFLOAT, "HIERARCH ESO TEL AMBI FWHM START", &dummyval, NULL, &status);
    fits_update_key(fptr, TFLOAT, "HIERARCH ESO TEL AMBI FWHM END", &dummyval, NULL, &status);
    fits_update_key(fptr, TFLOAT, "HIERARCH ESO TEL IA FWHM", &dummyval, NULL, &status);
    dummyval=0.0;
    fits_update_key(fptr, TFLOAT, "HIERARCH ESO TEL PARANG START", &dummyval, NULL, &status);
    fits_update_key(fptr, TFLOAT, "HIERARCH ESO TEL PARANG END", &dummyval, NULL, &status);
    fits_update_key(fptr, TFLOAT, "HIERARCH ESO TEL AIRMASS START", &dummyval, NULL, &status);
    fits_update_key(fptr, TFLOAT, "HIERARCH ESO TEL AIRMASS END", &dummyval, NULL, &status);
    fits_update_key(fptr, TFLOAT, "AIRMASS", &dummyval, NULL, &status);

    fits_update_key(fptr, TSTRING, "HIERARCH ESO OBS NAME","PRE-FAKE", NULL, &status);
    fits_update_key(fptr, TSTRING, "HIERARCH ESO INS SWSIM","PILMOSt", NULL, &status);

    /* Remove any CO keywords that may have been present in template just to be safe */
    for (ii=1; ii<14; ii++) {
      dummyval=0.0;
      sprintf(KW,"CO1_%d",ii);
      fits_update_key(fptr, TFLOAT, KW, &dummyval, NULL, &status);
      fits_delete_key(fptr, KW, &status);
      sprintf(KW,"CO2_%d",ii);
      fits_update_key(fptr, TFLOAT, KW, &dummyval, NULL, &status);
      fits_delete_key(fptr, KW, &status);
    }

    fits_update_key(fptr, TFLOAT, "HIERARCH ESO PRO INVCO1 RMS", &dummyval, NULL, &status);
    fits_delete_key(fptr, "HIERARCH ESO PRO INVCO1 RMS", &status);
    fits_update_key(fptr, TFLOAT, "HIERARCH ESO PRO INVCO2 RMS", &dummyval, NULL, &status);
    fits_delete_key(fptr, "HIERARCH ESO PRO INVCO2 RMS", &status);
    fits_update_key(fptr, TFLOAT, "HIERARCH ESO PRO CO1 RMS", &dummyval, NULL, &status);
    fits_delete_key(fptr, "HIERARCH ESO PRO CO1 RMS", &status);
    fits_update_key(fptr, TFLOAT, "HIERARCH ESO PRO CO2 RMS", &dummyval, NULL, &status);
    fits_delete_key(fptr, "HIERARCH ESO PRO CO2 RMS", &status);

    ra_up=targra+field_radius;
    ra_down=targra-field_radius;
    dec_up=targdec+field_radius;
    dec_down=targdec-field_radius;

    /*Set up the wcs structure, the -50 adjustment to CRPIX reflects the fact that reduced
      pre-images are cropped relative to raw exposures (the CRPIX in the database are for
      raw exposures*/
    vimoswcs=wcskinit (naxes[0], naxes[1], ctype1, ctype2, CRPIX1[quad-1]-50, CRPIX2[quad-1],
			    targra, targdec,NULL, SCALE, SCALE, -rotangle, 2000., 2000.);

    /*open and close entire cont. cat. file each time through*/
    /*First just get the column headings from the first row*/
    if (!(cont_cat_file=fopen(cont_cat_file_name,"r"))) {
      fprintf(stderr,"Couldn't find the contributed catalogue: %s\n",cont_cat_file_name);
      exit(0);
    }

    fgets(head_chars,255,cont_cat_file);
    ii=0;
    jj=0;
    mag_col=-1;
    col_cnt=0;
    last_head_char='Z';
    sprintf(outstring,"ID X_IMAGE Y_IMAGE RA DEC ");
    while (head_chars[ii]!='\0') {
      //      printf("char %d %c \n",ii,head_chars[ii]);
      if (head_chars[ii]==' ') {
	if (((strstr(cont_col_names[col_cnt],"mag")!=NULL && strstr(cont_col_names[col_cnt],"mage")==NULL) || (strstr(cont_col_names[col_cnt],"Mag")!=NULL) || (strstr(cont_col_names[col_cnt],"MAG")!=NULL && strstr(cont_col_names[col_cnt],"MAGE")==NULL)) && mag_col==-1) {
	  mag_col=col_cnt;
	}
	if (last_head_char!=' ') {
	  if(ii!=0) {
	    col_cnt++;
	    jj=0;
	  }
	  last_head_char=' ';
	}
      } else {
	if (col_cnt>2) { 
	  cont_col_names[col_cnt][jj]=head_chars[ii];
	  jj++;
	}
	last_head_char='Z';
      }
      ii++;
    }

    col_tot=col_cnt+1;
    fclose(cont_cat_file);
    for (col_cnt=3;col_cnt<col_tot;col_cnt++) {
      sprintf(outstring,"%s%s ",outstring,cont_col_names[col_cnt]);
    }
    //    sprintf(outstring,"%s%s",outstring,cont_col_names[col_cnt]);

    /*open and close entire cont. cat. file each time through*/
    /*Now get the rows with the data*/
    if (!(cont_cat_file=fopen(cont_cat_file_name,"r"))) {
      fprintf(stderr,"Couldn't find the contributed catalogue: %s\n",cont_cat_file_name);
      exit(0);
    }

    if (!(vm_cat_file=fopen(vm_cat_file_name,"w"))) {
      fprintf(stderr,"Couldn't write the VIMOS catalogue\n");
      exit(0);
    }

    fprintf(vm_cat_file,"%s ",outstring);

    fluxmin=1.0;
    row_cnt=0;
    /*check that we are not at the end of the file*/
    while (!feof(cont_cat_file)) {
      col_cnt=0;
      /*skip the header row*/
      if (row_cnt==0) {
	while(col_cnt<col_tot) {
	  fscanf(cont_cat_file,"%s",&col);
	  col_cnt++;
	}
      } else {
	while(col_cnt<col_tot) {
	  fscanf(cont_cat_file,"%s",&col);	
	  sprintf(cont_col_vals[col_cnt],"%s",col);
	  col_cnt++;
	}
	ID=atoi(cont_col_vals[0]);
	if (cont_col_vals[1][1]==':' || cont_col_vals[1][2]==':' || cont_col_vals[1][3]==':') {
	  RAsource=RAdeg2dec(cont_col_vals[1]);
	  DECsource=DECdeg2dec(cont_col_vals[2]);
	} else {
	  RAsource=atof(cont_col_vals[1]);
	  DECsource=atof(cont_col_vals[2]);
	}
	mag_in=atof(cont_col_vals[mag_col]);

	//printf("%d %lf %lf %lf \n",ID,RAsource,DECsource,mag_in);

	/*Ideally allow the chance to put in catalogue shift here??? (CAT_CORR_RA/DEC in python code)*/

	wcs2pix(vimoswcs, RAsource, DECsource, &x_basic, &y_basic, &offscl);
	//printf("%lf %lf %lf %lf \n", RAsource, DECsource, x_basic, y_basic);

	/*Then apply sky2ccd distortion correction, but only if on or near detector
	  the dist fit will diverge and it will be meaningless far from edges*/
        x_therm=-9999.9;
        y_therm=-9999.9;
	if (offscl==0 && x_basic>-10.0 && y_basic>-10.0 && x_basic<2200.0 && y_basic<2500.0) {
	  x_dist=0.0;
	  y_dist=0.0;
	  for (ii=0; ii<POLY_ORD; ii++) {
	    for (jj=0; jj<POLY_ORD; jj++) {
	      x_dist+=skyto_alpha[ii][jj]*pow(x_basic,jj)*pow(y_basic,ii);
	      y_dist+=skyto_beta[ii][jj]*pow(x_basic,jj)*pow(y_basic,ii);
	    }
	  }
	  /*Apply thermal correction*/
	  x_therm=(1.0+k_therm*(mask_temp[quad-1]-sky_temp[quad-1]))*(x_dist-xcen)+xcen;
	  y_therm=(1.0+k_therm*(mask_temp[quad-1]-sky_temp[quad-1]))*(y_dist-ycen)+ycen;
	}
	if (offscl==0 && x_therm>-10.0 && y_therm>-10.0 && x_therm<2200.0 && y_therm<2500.0) {
	  sprintf(outstring,"%d %lf %lf %lf %lf ",ID,x_therm,y_therm,RAsource,DECsource);
	  for (ii=3;ii<col_tot;ii++) {
	    sprintf(outstring,"%s %s",outstring,cont_col_vals[ii]);
	  }
	  fprintf(vm_cat_file,"%s \n",outstring);
	}
	if (x_therm>1.0 && x_therm<naxes[0]-1.5 && y_therm>1.0 && y_therm<naxes[1]-1.5 && mag_in!=0.0) {
	  x_frac=x_therm-(int)(x_therm);
	  y_frac=y_therm-(int)(y_therm);
	  flux=pow(10.0,(M0-mag_in)*0.4);
	  if (flux<fluxmin && mag_in<M0+5) fluxmin=flux;
	  /*split flux across 4 pixels*/
	  simple_array_float[(int)(y_therm)-1][(int)(x_therm)-1]+=flux*(1.0-x_frac)*(1.0-y_frac);
	  simple_array_float[(int)(y_therm)][(int)(x_therm)-1]+=flux*(1.0-x_frac)*y_frac;
	  simple_array_float[(int)(y_therm)-1][(int)(x_therm)]+=flux*x_frac*(1.0-y_frac);
	  simple_array_float[(int)(y_therm)][(int)(x_therm)]+=flux*x_frac*y_frac;
	}
      }
      row_cnt++;
    }
    printf("Wrote catalogue for quadrant %d\n",quad);
    /*convolve with PSF. This is purely to make the relative fluxes more apparent than they
      would be with just 1 pixel, it is in no way supposed to be a realistic simulation of
      the PSF*/
    datamax=0.0;
    for (ii=border;ii<naxes[1]-border;ii++) {
      for (jj=border;jj<naxes[0]-border;jj++) {
	sim_array[ii][jj]=0;
	for (kk=0;kk<9;kk++) {
	  for (mm=0;mm<9;mm++) {
	    if (simple_array_float[ii-4+kk][jj-4+mm]!=0.0) {
	      sim_array[ii][jj]+=(int)(simple_array_float[ii-4+kk][jj-4+mm]*psf[kk][mm]/fluxmin);
	    }
	  }
	}
	if (sim_array[ii][jj]>pix_sat) sim_array[ii][jj]=pix_sat;
	if (sim_array[ii][jj]>datamax) datamax=sim_array[ii][jj];
      }
    }

    /*interpolate between each vertex (get step size from sky2ccd transform)
      do transformation with sky2ccd linear transform for each quadrant and
      test for pix values between 0 and max, then simply add to sim image*/
    kk=0;
    endflag=0;

    while (vigra[0]!=0.0 && endflag!=1) {
      wcs2pix(vimoswcs, vigra[kk], vigdec[kk], &x_basic_vig1, &y_basic_vig1, &offscl1);
      if (vigra[kk+1]==0.0) {
	/*We have visited all the vertices, now link the last back to the first*/
	endflag=1;
	wcs2pix(vimoswcs, vigra[0], vigdec[0], &x_basic_vig2, &y_basic_vig2, &offscl2);
      } else {
	wcs2pix(vimoswcs, vigra[kk+1], vigdec[kk+1], &x_basic_vig2, &y_basic_vig2, &offscl2);
      }
      /*Then apply sky2ccd distortion correction, but only if on or near detector
	the dist fit will diverge and it will be meaningless far from edges*/
      x_dist_vig1=0.0;
      y_dist_vig1=0.0;
      x_dist_vig2=0.0;
      y_dist_vig2=0.0;
      for (ii=0; ii<POLY_ORD; ii++) {
	for (jj=0; jj<POLY_ORD; jj++) {
	  x_dist_vig1+=skyto_alpha[ii][jj]*pow(x_basic_vig1,jj)*pow(y_basic_vig1,ii);
	  y_dist_vig1+=skyto_beta[ii][jj]*pow(x_basic_vig1,jj)*pow(y_basic_vig1,ii);
	  x_dist_vig2+=skyto_alpha[ii][jj]*pow(x_basic_vig2,jj)*pow(y_basic_vig2,ii);
	  y_dist_vig2+=skyto_beta[ii][jj]*pow(x_basic_vig2,jj)*pow(y_basic_vig2,ii);
	}
      }
      if (offscl1==0 && x_basic_vig1>-10.0 && y_basic_vig1>-10.0 && x_basic_vig1<2200.0 && y_basic_vig1<2500.0) {
	/*Apply thermal correction and use distortion corrected co-ords*/
	gpx_pix=(1.0+k_therm*(mask_temp[quad-1]-sky_temp[quad-1]))*(x_dist_vig1-xcen)+xcen;
	gpy_pix=(1.0+k_therm*(mask_temp[quad-1]-sky_temp[quad-1]))*(y_dist_vig1-ycen)+ycen;
      } else {
	/*Otherwise just use the WCS prediction, this is still useful for
	  drawing the approx vignette*/
	gpx_pix=x_basic_vig1;
	gpy_pix=y_basic_vig1;
      }
      if (offscl2==0 && x_basic_vig2>-10.0 && y_basic_vig2>-10.0 && x_basic_vig2<2200.0 && y_basic_vig2<2500.0) {
	/*Apply thermal correction and use distortion corrected co-ords*/
	gpx_pix_next=(1.0+k_therm*(mask_temp[quad-1]-sky_temp[quad-1]))*(x_dist_vig2-xcen)+xcen;
	gpy_pix_next=(1.0+k_therm*(mask_temp[quad-1]-sky_temp[quad-1]))*(y_dist_vig2-ycen)+ycen;
      } else {
	/*Otherwise just use the WCS prediction, this is still useful for
	  drawing the approx vignette*/
	gpx_pix_next=x_basic_vig2;
	gpy_pix_next=y_basic_vig2;
      }
      deltax=gpx_pix_next-gpx_pix;
      deltay=gpy_pix_next-gpy_pix;
      /*Set number of steps to twice the number required to have 1 per pix*/
      if (abs(deltax)>abs(deltay)) {
	nstep=abs(deltax*2.0);
      } else {
	nstep=abs(deltay*2.0);
      }
      for (jj=0;jj<nstep;jj++) { 
	gpx_interp=gpx_pix+jj*deltax/(float)(nstep);
	gpy_interp=gpy_pix+jj*deltay/(float)(nstep);
	if (gpx_interp>1.0 && gpx_interp<naxes[0]-1.5 && gpy_interp>1.0 && gpy_interp<naxes[1]-1.5) {
	  /* Add large value to sim image */
	  sim_array[(int)(gpy_interp)][(int)(gpx_interp)]=100000.0;
	}
      }
      kk++;
    }

    for (ii=0;ii<naxes[1];ii++) {
      sim_array[ii][1]=100000;
      sim_array[ii][naxes[0]-2]=100000;
    }
    for (jj=0;jj<naxes[0];jj++) {
      sim_array[IFU_vignette_rows][jj]=100000;
      sim_array[naxes[1]-IFU_vignette_rows][jj]=100000;
    }

    if (quad==1 || quad==2) {
      corner_1x=0;
      corner_1y=naxes[1]-corner_vignette[1];
      corner_2x=corner_vignette[0];
      corner_2y=naxes[1]-1;
    }
    else {
      corner_1x=0;
      corner_1y=corner_vignette[1];
      corner_2x=corner_vignette[0];
      corner_2y=0;
    }

    deltax=corner_2x-corner_1x;
    deltay=corner_2y-corner_1y;
    if (abs(deltax)>abs(deltay)) {
      nstep=abs(deltax*2.0);
    } else {
      nstep=abs(deltay*2.0);
    }
    for (jj=0;jj<nstep;jj++) { 
      x_interp=corner_1x+jj*deltax/(float)(nstep);
      y_interp=corner_1y+jj*deltay/(float)(nstep);
      if (x_interp>1.0 && x_interp<naxes[0]-1.5 && y_interp>(float)(IFU_vignette_rows) && y_interp<naxes[1]-(float)(IFU_vignette_rows)) {
	/* Add large value to sim image */
	sim_array[(int)(y_interp)][(int)(x_interp)]=100000.0;
      }
      x_interp_mirror=naxes[0]-x_interp;
      if (x_interp_mirror>1.0 && x_interp_mirror<naxes[0]-1.5 && y_interp>(float)(IFU_vignette_rows) && y_interp<naxes[1]-(float)(IFU_vignette_rows)) {
	/* Add large value to sim image */
	sim_array[(int)(y_interp)][(int)(x_interp_mirror)]=100000.0;
      }
    }


    datamin=0.0;
    fits_update_key(fptr, TINT,"DATAMIN", &datamin, NULL, &status);
    fits_update_key(fptr, TINT,"DATAMAX", &datamax, NULL, &status);
    status=0;
    fclose(vm_cat_file);

    fits_write_img(fptr, TINT, fpixel, (naxes[0]*naxes[1]), sim_array[0], &status);
    if (status != 0)
      printf("ERROR: fits_write_img, status=%d\n", status);
    fits_close_file(fptr, &status);
    if (status != 0) {
      printf("ERROR: fits_close_file, status=%d\n", status);
    } else {
      printf("Wrote simulated pre-image for quadrant %d\n",quad);
    }
    fits_report_error(stderr, status);
    free(vimoswcs);
    fclose(cont_cat_file);
  }
  return 0;
}

/*___PILMOS___*/
