/*******************************************************************************
# E.S.O. - VLT Project
#
# "@(#) $Id: PILMOSt.h 229096 2012-06-05 17:59:04Z pbaksai $"
#
# who       when        what
# --------  ----------  --------------------------------------------------------
# pbaksai   2012/05/21  CCB-00246: Updated with latest coeffs. from 2012/05/21
# pbaksai   2012/05/20  Added this header
# pbristow  2012/02/28  Created
#
#*******************************************************************************
#  NAME
#      PILMOSt.h - header file for main program PILMOSt.c
#
#  SYNOPSIS
#      #include "PILMOSt.h"
#
#  DESCRIPTION
#
#  PARENTS (Inheritance)
#      None
#      
#  PEERS (Composition)
#      None
#
#  ARGUMENTS
#
#  PUBLIC METHODS
#      
#  PUBLIC VARIABLES
#
#  PARAMETERS
#
#  INPUTS
#
#  OUTPUTS
#
#  ENVIRONMENT
#
#  RETURN VALUES
#
#  CAUTIONS
#      Source of coefficients should be a file PILMOS_coeff.h
#      distributed by SciOps every six months.
#      It is included in this file near the bottom.
#
#  EXAMPLES
#
#  SEE ALSO
#
#
#*******************************************************************************
0        1         2         3         4         5         6         7         8
123456789 123456789 123456789 123456789 123456789 123456789 123456789 1234567890
*/
#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include "fitsio.h" 
#include "fitsio2.h" 
#include "wcs.h"

/*PROTOTYPES*/
int parse_gcline(char *name, char*namecheck);
double RAdeg2dec(char *RAdeg);
double DECdeg2dec(char *DECdeg);

/*3rd order bivariate polys are used throughout VIMOS software*/
#define POLY_ORD 4

static char sim_root_name[6]="pilmos";

/*VIMOS data always uses the following CTYPEs*/
char* ctype1="RA---TAN";
char* ctype2="DEC--TAN";

/*Arbitrary zeropoint for scaling intensities in fake images*/
double M0=22.5;
int pix_sat=65535;

/*Array size (after basic pipeline reduction) */
static int SIZE_A=2048;
static int SIZE_B=2440;
static int border=5; /*must not be <5 or convolution with psf will fail*/
static double field_radius=0.3;

/*thermal expansion parameters*/
static double k_therm=-0.00008;
static double xcen=1024.0;
static double ycen=1220.0;

#
static double corner_vignette[2]={350.0,250.0};
static int IFU_vignette_rows=2;

/*simple, arbitrary PSF*/
static double psf[9][9]={{ 0.00,  0.00,  0.01,  0.05,  0.10,  0.05,  0.01,  0.00,  0.00},
			 { 0.00,  0.02,  0.05,  0.10,  0.20,  0.10,  0.05,  0.02,  0.00},
			 { 0.01,  0.05,  0.20,  0.30,  0.50,  0.30,  0.20,  0.05,  0.01},
			 { 0.05,  0.10,  0.30,  0.80,  0.90,  0.80,  0.30,  0.10,  0.05},
			 { 0.10,  0.20,  0.50,  0.90,  1.00,  0.90,  0.50,  0.20,  0.10},
			 { 0.05,  0.10,  0.30,  0.80,  0.90,  0.80,  0.30,  0.10,  0.05},
			 { 0.01,  0.05,  0.20,  0.30,  0.50,  0.30,  0.20,  0.05,  0.01},
			 { 0.00,  0.02,  0.05,  0.10,  0.20,  0.10,  0.05,  0.02,  0.00},
			 { 0.00,  0.00,  0.01,  0.05,  0.10,  0.05,  0.01,  0.00,  0.00}};


/*******************************************************************************
 * WCS params, SKY2CCD coeffs and CCD2MASK coeffs below should be
 * updated every 6 months AND KEPT CONSISTENT
 ******************************************************************************/

#include "PILMOSt_coeffs.h"

/*******************************************************************************
 * End of WCS params, SKY2CCD coeffs and CCD2MASK coeffs block
 ******************************************************************************/

/*___oOo___*/
